<?php
	if (!defined("ENV")) exit();

	pageHeader("Search Engine Settings", "", "");
	pageNote(array(
		array("icon"=>"ic_hint.gif", "text"=>"<b>Before enabling or regenerating flat URLs, keep in mind the format in which current pages are indexed.</b>", "url"=>"", "visible"=>true)
	));
	
	$action = empty($action)?"":$action;
	$is_htaccess_error = false;
	$is_robots_error = false;
	if (in_array($action, array("update", "force-htaccess", "force-url")) && !DEMO_LOCK)
	{
		/**
		 * Save robots.txt file
		 */
		if ($action == "update")
		{
			$is_robots_error = !file_put_contents("robots.txt", $robots);
		}
		
		/**
		 * Create .htaccess file
		 */
		if (in_array($action, array("update", "force-htaccess")) && !DEMO_LOCK)
		{
			@unlink(".htaccess.back");
			@copy(".htaccess", ".htaccess.back");
			$f = @fopen(".htaccess", "w");
			if ($f)
			{
				$pu = parse_url("http://foo.com/".$_SERVER["REQUEST_URI"]);
				$pi = pathinfo($pu["path"]);
				$rewriteBase = $pi["dirname"];
			
				$cond = '\#(.)*|\?(.)*';
				$d = dir($settings["GlobalServerPath"]);
				while (false !== ($entry = $d->read()))
				{
					if (!in_array($entry, array(".", "..")))
					{
		   				if (is_file($entry))
   						{
   							$cond .= ($cond == "" ? "" : "|").(str_replace(".", "\\.", $entry)."(.)*");
   						}
   						elseif (is_dir($entry))
   						{
   							$cond .= ($cond == "" ? "" : "|").($entry."\\/(.)*");
   						}
					}
				}
				$d->close();
				
				$form["USE_MOD_REWRITE"] = isset($form["USE_MOD_REWRITE"]) ? $form["USE_MOD_REWRITE"] : $settings["USE_MOD_REWRITE"];
				
				$htaccess = 
				
					"<IfModule mod_headers.c>\n".
					"\tHeader unset ETag\n".
					"\tFileETag None\n".
					"\t<FilesMatch \"(?i)^.*\.(ico|flv|jpg|jpeg|png|gif|js|css)$\">\n".
					"\t\tHeader unset Last-Modified\n".
					"\t\tHeader set Expires \"Fri, 21 Dec 2020 00:00:00 GMT\"\n".
					"\t\tHeader set Cache-Control \"public, no-transform\"\n".
					"\t</FilesMatch>\n".
					"</IfModule>\n\n".
                     
					"<IfModule mod_rewrite.c>\n".
					"\tRewriteEngine\t".($form["USE_MOD_REWRITE"]=="YES"?"On":"Off")."\n".
					"\tRewriteBase\t".str_replace("//", "/", $rewriteBase)."\n".
					
					// 2010-08-02 pkolas * custom rules
					'
					RewriteRule	^gallery(\d+).xml$	galleryxml.php?pid=$1 [L]
					RewriteRule	^video(\d+).xml$	videoxml.php?pid=$1 [L]
					RewriteCond	$1 !^(galleryxml\.php(.)*|videoxml\.php(.)*|calc\.php(.)*|box_quote\.php(.)*|getpdf\.php(.)*|(.+)\.swf(.)*|(.+)\.xml(.)*|(.+)\.f4v(.)*|covers(.+)\.jpg(.)*)
					'.
					// 2010-08-02 pkolas end
					
					//"RewriteCond\t$1 !^(content\/(.)*|images\/(.)*|install\/(.)*|admin\.php(.)*|download\.php(.)*|index\.php(.)*|login\.php(.)*|\#(.)*|\?(.)*)\n".
					"\tRewriteCond\t$1 !^(".$cond.")\n".
					"\tRewriteRule\t^(.+)$ index.php?url=$1 [L]\n".
					"</IfModule>\n\n".
				
					"<IfModule mod_deflate.c>\n".
					"\t<FilesMatch \"\.(js|css|ico|flv|jpg|jpeg|png|gif)$\">\n".
					"\t\tSetOutputFilter DEFLATE\n".
					"\t</FilesMatch>\n".
					"</IfModule>\n";
				
				if(!@fputs($f, $htaccess, strlen($htaccess))){
					$is_htaccess_error = true;
				}
				fclose($f);
			}
			else
			{
				$is_htaccess_error = true;
				$form["USE_MOD_REWRITE"] = "NO";
			}
		}
		
		/**
		 * Update settings in DB
		 */
		if ($action == "update" && !DEMO_LOCK)
		{
			foreach($form as $var=>$value)
			{
				$db->reset();
				$db->assignStr("value", $value);
				$db->update(DB_PREFIX."settings", "WHERE name='".$db->escape($var)."'");
				$settings[$var] = $value;
			}
		}
		
		/**
		 * Generate flat URLs
		 */
		if (in_array($action, array("update", "force-url")) && !DEMO_LOCK)
		{
			//check are there changes in URL templates
			if (isset($_SESSION["seo-settings"]))
			{
				//category template changed
				$updateCategories = (
					$_SESSION["seo-settings"]["SearchURLCategoryTemplate"] != $settings["SearchURLCategoryTemplate"] ||
					$_SESSION["seo-settings"]["SearchAutoGenerateLowercase"] != $settings["SearchAutoGenerateLowercase"] ||
					$_SESSION["seo-settings"]["SearchURLJoiner"] != $settings["SearchURLJoiner"] || 
					$_SESSION["seo-settings"]["USE_MOD_REWRITE"] != $settings["USE_MOD_REWRITE"] ||
					$action == "force-url"
				);
			
				//product template changed
				$updateProducts = (
					$_SESSION["seo-settings"]["SearchURLProductTemplate"] != $settings["SearchURLProductTemplate"] ||
					$_SESSION["seo-settings"]["SearchAutoGenerateLowercase"] != $settings["SearchAutoGenerateLowercase"] ||
					$_SESSION["seo-settings"]["SearchURLJoiner"] != $settings["SearchURLJoiner"] ||
					$_SESSION["seo-settings"]["USE_MOD_REWRITE"] != $settings["USE_MOD_REWRITE"] ||
					$action == "force-url"
				);
			
				//page template changed
				$updatePages = (
					$_SESSION["seo-settings"]["SearchURLPageTemplate"] != $settings["SearchURLPageTemplate"] ||
					$_SESSION["seo-settings"]["SearchAutoGenerateLowercase"] != $settings["SearchAutoGenerateLowercase"] ||
					$_SESSION["seo-settings"]["SearchURLJoiner"] != $settings["SearchURLJoiner"] ||
					$_SESSION["seo-settings"]["USE_MOD_REWRITE"] != $settings["USE_MOD_REWRITE"] ||
					$action == "force-url"
				);
				
				updateSeoURLs($db, $settings, $updateCategories, $updateProducts, $updatePages);
			}
		}
	}
	if($is_htaccess_error || $is_robots_error)
	{
		pageNote(array(
			array("icon"=>"ic_error.gif", "text"=>"<font color=\"#990000\"><b>Please note: Error writing '.htaccess' file. Please check file permissions.</b></font>", "url"=>"", "visible"=>$is_htaccess_error),
			array("icon"=>"ic_error.gif", "text"=>"<font color=\"#990000\"><b>Please note: Error writing 'robots.txt' file. Please check file permissions.</b></font>", "url"=>"", "visible"=>$is_robots_error)
		));
	}
	
	$f = @fopen("robots.txt", "r");
	if ($f)
	{
		$robots = "";
		while(($s = fgets($f, 1024)) != false) $robots.=$s;
		fclose($f);
	}
	
	$_SESSION["seo-settings"] = array(
		"SearchURLCategoryTemplate" => $settings["SearchURLCategoryTemplate"],
		"SearchAutoGenerateLowercase" => $settings["SearchAutoGenerateLowercase"],
		"SearchURLProductTemplate" => $settings["SearchURLProductTemplate"],
		"SearchURLPageTemplate" => $settings["SearchURLPageTemplate"],
		"SearchURLJoiner" => $settings["SearchURLJoiner"],
		"USE_MOD_REWRITE" => $settings["USE_MOD_REWRITE"]
	);
	pageNote(array(
		array("icon"=>"ic_repair.gif", "text"=>"Regenerate .htaccess file", "url"=>"admin.php?p=settings_search&action=force-htaccess", "visible"=>true),
		array("icon"=>"ic_repair.gif", "text"=>"Regenerate flat URLs", "url"=>"admin.php?p=settings_search&action=force-url", "visible"=>true)
	));
	pageTabs(array(
		array("title"=>"Search Engine Settings", "id"=>"Main", "active"=>true, "width"=>140),
	));
?>
<form name="frmSettings" action="admin.php?p=settings_search" method="post">
	<input type="hidden" name="action" value="update">
<div id="idTabSheetMain" class="sheetActive">
	<table width="100%" cellpadding="0" cellspacing="5" border="0">
		<tr>
			<td class="formHeader" colspan="2">
				<div class="formHeaderIcon"><img src="images/icons/ic_settings.gif"/></div>
				<div class="formHeaderText">Search Engine Settings</div>
			</td>
		</tr>
		<tr valign="top">
			<td class="formItemCaption">Page (Site) Title</td>
			<td class="formItemControl">
				<input type="text" name="form[SearchTitle]" style="width:400px;" value="<?=htmlspecialchars($settings["SearchTitle"])?>">
				<div class="formItemComment">
					Site title tag (&lt;title&gt; tag), should not be more than 60 characters.
				</div>
			</td>
		</tr>
		
		<tr valign="top">
			<td class="formItemCaption">Enable flat URLs generation?</td>
			<td class="formItemControl">
				<?if(_ACCESS_ALL || _ACCESS_FLAT_URL){?>
				<select name="form[USE_MOD_REWRITE]" style="width:123px;">
					<option value="YES">Yes</option>
					<option value="NO" <?=$settings["USE_MOD_REWRITE"]=="NO"?"selected":""?>>No</option>
				</select>
				<?}else echo '<div style="padding-top:4px;"><b>This feature is not available under your license agreement. Please contact your provider to upgrade your application.</b></div>';?>
				<div class="formHelp">Please note: this feature requires MOD_REWRITE Apache Web server Module and does not work on all servers.</div>
			</td>
		</tr>
		
		<tr valign="top">
			<td class="formItemCaption">Lowercase generated URLs?</td>
			<td class="formItemControl">
				<select name="form[SearchAutoGenerateLowercase]" style="width:123px;">
					<option value="YES">Yes</option>
					<option value="NO" <?=$settings["SearchAutoGenerateLowercase"]=="NO"?"selected":""?>>No</option>
				</select>
			</td>
		</tr>
		
		<tr valign="top">
			<td class="formItemCaption">URL words joiner</td>
			<td class="formItemControl">
				<select name="form[SearchURLJoiner]" style="width:123px;">
					<option value="-">- (dash)</option>
					<option value="_" <?=$settings["SearchURLJoiner"]=="_"?"selected":""?>>_ (underscore)</option>
					<option value="" <?=$settings["SearchURLJoiner"]==""?"selected":""?>>none</option>
				</select>
			</td>
		</tr>
		
		<tr valign="top">
			<td class="formItemCaption">Category page URL template</td>
			<td class="formItemControl">
				<input type="text" name="form[SearchURLCategoryTemplate]" style="width:400px;" value="<?=htmlspecialchars($settings["SearchURLCategoryTemplate"])?>">
				<div class="formItemComment">
					Template Variables:<br/>
					%CategoryKey%, %CategoryName%, %CategoryDbId%, %CategoryPath% - at least one variable required<br/>
				</div>
			</td>
		</tr>
		<tr valign="top">
			<td class="formItemCaption">Category page pagination</td>
			<td class="formItemControl">
				<input type="text" name="form[SearchURLCategoryPaginatorTemplate]" style="width:400px;" value="<?=htmlspecialchars($settings["SearchURLCategoryPaginatorTemplate"])?>">
				<div class="formItemComment">
					Template Variables:<br/>
					%PageNumber% - required<br/>
					Will be added before extension (if exists) or closing slash symbol. Resulting URL will look like:<br/>
					[category-url][page-number].html<br/>
					[category-url][page-number]<br/>
					[category-url][page-number]/<br/>
				</div>
			</td>
		</tr>
		<tr valign="top">
			<td class="formItemCaption">Product page URL template</td>
			<td class="formItemControl">
				<input type="text" name="form[SearchURLProductTemplate]" style="width:400px;" value="<?=htmlspecialchars($settings["SearchURLProductTemplate"])?>">
				<div class="formItemComment">
					Template variables:<br/>
					%CategoryKey%, %CategoryName%, %CategoryDbId%, %CategoryPath%<br/>
					%ProductId%, %ProductName%, %ProductDbId% - at least one of product variables is required<br/>
				</div>
			</td>
		</tr>
		<tr valign="top">
			<td class="formItemCaption">Text page URL template</td>
			<td class="formItemControl">
				<input type="text" name="form[SearchURLPageTemplate]" style="width:400px;" value="<?=htmlspecialchars($settings["SearchURLPageTemplate"])?>">
				<div class="formItemComment">
					Template variables:<br/>
					%PageName%, %PageTitle%
				</div>
			</td>
		</tr>
		
		<tr valign="top">
			<td class="formItemCaption">Allow keyword auto-generation?</td>
			<td class="formItemControl">
				<select name="form[SearchAutoGenerate]" style="width:123px;">
					<option value="YES">Yes</option>
					<option value="NO" <?=$settings["SearchAutoGenerate"]=="NO"?"selected":""?>>No</option>
				</select>			
			</td>
		</tr>
	
		<tr valign="top">
			<td class="formItemCaption">Meta Keywords</td>
			<td class="formItemControl">
				<textarea class="no-mce" cols="50" rows="8" style="width:100%;height:80px;" name="form[SearchMetaKeywords]"><?=htmlspecialchars($settings["SearchMetaKeywords"])?></textarea>
				<div class="formItemComment">Recommended length is 200-1000 characters. Separate keywords and phrases by using a comma</div>
			</td>
		</tr>
		<tr valign="top">
			<td class="formItemCaption">Meta Description</td>
			<td class="formItemControl">
				<textarea class="no-mce" cols="50" rows="3" style="width:100%;height:80px;" name="form[SearchMetaDescription]"><?=htmlspecialchars($settings["SearchMetaDescription"])?></textarea>
				<div class="formItemComment">Recommended length is 6-125 characters.</div>
			</td>
		</tr>
		<tr valign="top">
			<td class="formItemCaption">File "robots.txt"</td>
			<td class="formItemControl">
				<textarea class="no-mce" cols="50" wrap="off" rows="8" style="width:100%;height:80px;" name="robots"><?=htmlspecialchars($robots)?></textarea>
				<div class="formItemComment">This file creates a robots.txt file that will be located in your main directory. This file makes it easier for the search engines to index your website.</div>
			</td>
		</tr>
	</table>
</div>
<div class="formButtons">
<input type="image" src="images/admin/bt_save_changes.gif"> &nbsp;
			<input type="image" src="images/admin/bt_reset_form.gif" onClick="frmSettings.reset();return false;">
</div>
</form>	
