<?php
if(!class_exists("nusoap_client")) require_once("content/vendors/nusoap/nusoap.php");

/**
 * buySAFE API Class
 *
 */
class BuySafeAPI
{
	public $test_mode = false;
	public $hash = "";
	public $version = "610";
	public $username = "";
	public $password = "";
	public $token = "";
	public $api_url = "";
	
	private $_xmlns = "http://ws.buysafe.com";
	
	public $last_error = false;
	
	/**
	 * Class constructor
	 *
	 * @param array $settings
	 * @return BuySafeAPI
	 */
	public function __construct(&$settings)
	{
		$this->test_mode = $settings["buysafe_mode"] == "Test";
		$this->hash = $settings["buysafe_hash"];
		$this->username = $settings["buysafe_username"];
		$this->password = $settings["buysafe_password"];
		$this->token = $settings["buysafe_token"];
		$this->api_url = $this->test_mode ? "https://sbws.buysafe.com/BuysafeWS/CheckoutAPI.dll" : "https://api.buysafe.com/BuysafeWS/CheckoutAPI.dll";
		
		return $this;
	}
	
	/**
	 * Connect to buySafe server and execute API call
	 *
	 * @param string $body
	 */
	public function call($action, $soap_body)
	{
		$soap_header = 
			'<BuySafeWSHeader xmlns="'.$this->_xmlns.'">'."\n".
 				'<Version>'.utf8_encode($this->version).'</Version>'."\n".
			'</BuySafeWSHeader>'."\n".
			'<MerchantServiceProviderCredentials xmlns="'.$this->_xmlns.'">'."\n".
				'<UserName>'.utf8_encode($this->username).'</UserName>'."\n".
				'<Password>'.utf8_encode($this->password).'</Password>'."\n".
 			'</MerchantServiceProviderCredentials>'."\n".
			'<BuySafeUserCredentials xmlns="'.utf8_encode($this->_xmlns).'">'."\n".
 				'<AuthenticationTokens>'."\n".
					'<string>'.utf8_encode($this->token).'</string>'."\n".
				'</AuthenticationTokens>'."\n".
 			'</BuySafeUserCredentials>'."\n";
 		
 		$soap = new nusoap_client($this->api_url);
 		
 		if ($err = $soap->getError())
 		{
 			$this->last_error = $err;
 			return false;
 		}
 		
 		$soap->soap_defencoding='UTF-8';
 		
 		$response = $soap->call($action, $soap_body, $this->_xmlns, "http://ws.buysafe.com/".$action, $soap_header, null, 'rpc', 'literal');
 	
 		if ($err = $soap->getError())
 		{
 			$this->last_error = $err;
 			return false;
 		}
 		
 		return $response;
	}
	
	/**
	 * Add / update shopping cart call
	 *
	 * @param object $user
	 * @param object $cart
	 */
	public function addUpdateShoppingCart($user, $order, $db)
	{
		
		$billing = $user->auth_ok ? $user->data : false;
		
		if(isset($order->items) && (count($order->items) > 0))
		{
			$body = 
				'<AddUpdateShoppingCart xmlns="'.$this->_xmlns.'">'."\n".
 					'<ShoppingCartAddUpdateRQ>'."\n".
 						'<BuyerInfo>'."\n".
 							'<FirstName>'.utf8_encode(str_replace("&", "&amp;", $billing ? $billing["fname"] : ""))."</FirstName>\n".
 							'<LastName>'.utf8_encode(str_replace("&", "&amp;", $billing ? $billing["lname"] : ""))."</LastName>\n".
 							'<Email>'.utf8_encode(str_replace("&", "&amp;", $billing ? $billing["email"] : ""))."</Email>\n".
	 						'<Type>Unknown</Type>'."\n".
 							'<BillingAddress>'."\n".
 								'<PostalCode>'.utf8_encode(str_replace("&", "&amp;", $billing ? $billing["zip"] : ""))."</PostalCode>\n".
 								'<CountryCode>'.utf8_encode(str_replace("&", "&amp;", $billing ? $billing["country_iso_a2"] : ""))."</CountryCode>\n".
 								'<CountryCode>US</CountryCode>'."\n".
 							'</BillingAddress>'."\n".
 							//'<ShippingAddress>'."\n".
 							//	'<CountryCode>US</CountryCode>'."\n".
 							//'</ShippingAddress>'."\n".
	 					'</BuyerInfo>'."\n".
 						'<ShoppingCartId>PinnacleShoppingCart</ShoppingCartId>'."\n".
 						'<ClientIP>'.utf8_encode($_SERVER['REMOTE_ADDR']).'</ClientIP>'."\n".
						'<Items>'."\n";
			foreach($order->items as $item){

				$body.=		'<ShoppingCartItem>'."\n".
								'<UserToken>'.utf8_encode($this->token).'</UserToken>'."\n".
 								'<Title>'.utf8_encode(str_replace("&", "&amp;", $item["title"])).'</Title>'."\n". 								
 								'<Attributes>'.utf8_encode(str_replace("&", "&amp;", $item["options"])).'</Attributes>'."\n".
 								'<MarketplaceItemCode>'.utf8_encode(str_replace("&", "&amp;", $item["product_id"].($item["product_sub_id"] != "" ? (" - ".$item["product_sub_id"]) : ""))).'</MarketplaceItemCode>'."\n".
 								'<QuantityPurchased>'.utf8_encode($item["quantity"]).'</QuantityPurchased>'."\n".
 								'<PriceInfo>'."\n".
 									'<FinalPrice>'."\n".
	 									'<Value>'.utf8_encode(number_format($item["product_price"], 2, ".", "")).'</Value>'."\n".
 										'<CurrencyCode>USD</CurrencyCode>'."\n".
									'</FinalPrice>'."\n".
								'</PriceInfo>'."\n".
 								'<URLInfo>'."\n".
 									'<ViewItem>'.utf8_encode(getProductUrl($item["url"], $item["pid"], $item["cid"])).'</ViewItem>'."\n".
 									($item["image_url"] != "" ? ('<Thumbnail>'.utf8_encode($item["image_url"]).'</Thumbnail>'."\n") : "").
								'</URLInfo>'.
								'<isRestricted>'.(($item["digital_product"] == "Yes") || ($item["product_price"] == 0)? "true" : "false").'</isRestricted>'."\n".
							'</ShoppingCartItem>'."\n";
			}
			$body .=	'</Items>'."\n".
 						'<WantsBond>'."\n".
 							'<Value>'.($order->buysafeWantsBond == "true" ? "true" : "false").'</Value>'."\n".
 							'<HasBoolean>'.($order->buysafeWantsBond == "" ? "false" : "true").'</HasBoolean>'."\n".
 						'</WantsBond>'."\n".
	 				'</ShoppingCartAddUpdateRQ>'."\n".
 				'</AddUpdateShoppingCart>';
 		
 		
	 		if ($result = $this->call("AddUpdateShoppingCart", $body))
 			{
	
				$db->reset();
				$db->assignStr("buysafe_wants_bond", $result["BondCostDisplayText"] == "" ? "false" : "true");
				$db->assignStr("buysafe_bond_amount", $result["TotalBondCost"]);
				$db->update(DB_PREFIX."orders", "WHERE oid=".$order->oid);
				//echo $db->query;
 			}
 		
 			return $result;
		}
		return false;
	}
	
	/**
	 * Shopping cart checkout
	 *
	 * @param object $user
	 * @param object $cart
	 */
	public function setShoppingCartCheckout($user, $order, $db)
	{
		$billing = $user->data;
		$shipping = isset($order->shippingAddress) ? $order->shippingAddress : false;
		$body = 
			'<SetShoppingCartCheckout xmlns="http://ws.buysafe.com">'.
 				'<ShoppingCartCheckoutRQ xmlns="http://ws.buysafe.com">'.
 					'<ShoppingCartId>PinnacleShoppingCart</ShoppingCartId>'.
 					'<OrderNumber>'.utf8_encode($order->oid).'</OrderNumber>'.
 					'<ClientIP>'.utf8_encode($_SERVER['REMOTE_ADDR']).'</ClientIP>'."\n".
 					'<BuyerInfo>'.
 						'<FirstName>'.utf8_encode(str_replace("&", "&amp;", $billing["fname"])).'</FirstName>'.
						'<LastName>'.utf8_encode(str_replace("&", "&amp;", $billing["lname"])).'</LastName>'.
 						'<Email>'.utf8_encode(str_replace("&", "&amp;", $billing["email"])).'</Email>'.
 						'<BuyerType>Unknown</BuyerType>'.
 						'<BillingAddress>'.
							'<PostalCode>'.utf8_encode(str_replace("&", "&amp;", $billing["zip"])).'</PostalCode>'.
							'<CountryCode>'.utf8_encode(str_replace("&", "&amp;", $billing["country_iso_a2"])).'</CountryCode>'.
						'</BillingAddress>'.
						'<ShippingAddress>'.
							'<PostalCode>'.utf8_encode(str_replace("&", "&amp;", ($shipping ? $shipping["zip"] : "none"))).'</PostalCode>'.
							'<CountryCode>'.utf8_encode(str_replace("&", "&amp;", ($shipping ? $shipping["country_iso_a2"] : "none"))).'</CountryCode>'.
						'</ShippingAddress>'.
					'</BuyerInfo>'.
					'<WantsBond>'."\n".
 						'<Value>'.($order->buysafeWantsBond == "true" ? "true" : "false").'</Value>'."\n".
 						'<HasBoolean>'.($order->buysafeWantsBond == "" ? "false" : "true").'</HasBoolean>'."\n".
 					'</WantsBond>'."\n".
					'<Items>'."\n";
		foreach($order->items as $item){
			$body.=		'<ShoppingCartItem>'."\n".
							'<UserToken>'.utf8_encode($this->token).'</UserToken>'."\n".
 							'<Title>'.utf8_encode(str_replace("&", "&amp;", $item["title"])).'</Title>'."\n".
 							'<Attributes>'.utf8_encode(str_replace("&", "&amp;", $item["options"])).'</Attributes>'."\n".
 							'<MarketplaceItemCode>'.utf8_encode(str_replace("&", "&amp;", $item["product_id"].($item["product_sub_id"] != "" ? (" - ".$item["product_sub_id"]) : ""))).'</MarketplaceItemCode>'."\n".
 							'<QuantityPurchased>'.utf8_encode($item["quantity"]).'</QuantityPurchased>'."\n".
 							'<PriceInfo>'."\n".
 								'<FinalPrice>'."\n".
 									'<Value>'.utf8_encode(number_format($item["product_price"], 2, ".", "")).'</Value>'."\n".
 									'<CurrencyCode>USD</CurrencyCode>'."\n".
								'</FinalPrice>'."\n".
							'</PriceInfo>'."\n".
 							'<URLInfo>'."\n".
 								'<ViewItem>'.utf8_encode(getProductUrl($item["url"], $item["pid"], $item["cid"])).'</ViewItem>'."\n".
 								($item["image_url"] != "" ? ('<Thumbnail>'.utf8_encode($item["image_url"]).'</Thumbnail>'."\n") : "").
							'</URLInfo>'."\n".
							'<isRestricted>'.(($item["digital_product"] == "Yes") || ($item["product_price"] == 0)? "true" : "false").'</isRestricted>'."\n".
						'</ShoppingCartItem>'."\n";
		}
		$body .=	'</Items>'."\n".
				'</ShoppingCartCheckoutRQ>'.
			'</SetShoppingCartCheckout>';
			
		if ($result = $this->call("SetShoppingCartCheckout", $body))
 		{
			$db->reset();
			$db->assignStr("buysafe_wants_bond", "false");
			$db->update(DB_PREFIX."orders", "WHERE oid=".$order->oid);
			
			return $result;
 		}
 		
 		return false;
	}
	
	/**
	 * Cancel order
	 *
	 * @return unknown
	 */
	public function setShoppingCartCancelOrder($oid)
	{
		$body = 
			'<SetShoppingCartCancelOrder xmlns="http://ws.buysafe.com">'."\n".
				'<SetShoppingCartCancelOrderRQ>'."\n".
					'<IPAddress>'.utf8_encode($_SERVER['REMOTE_ADDR']).'</IPAddress>'."\n".
					'<DateProcessed>'."\n".
						'<DateTimeValue>0001-01-01T00:00:00</DateTimeValue>'."\n".
						'<HasDateTime>false</HasDateTime>'."\n".
					'</DateProcessed>'."\n".
					'<PaymentProcessed>false</PaymentProcessed>'."\n".
					'<ShoppingCartId>'.intval($oid).'</ShoppingCartId>'."\n".
				'</SetShoppingCartCancelOrderRQ>'."\n".
			'</SetShoppingCartCancelOrder>'."\n";
			
		if ($result = $this->call("SetShoppingCartCancelOrder", $body))
 		{
			return $result;
 		}
 		
 		return false;
	}
}