<?php 

/**
 * @class ShoppingCartCatalog
 */
class ShoppingCartCatalog{
	public $db;
	public $catalog_images_dir = "images/catalog";
	public $tree = array();
	
	/* CONSTRUCTOR
	*/
	function ShoppingCartCatalog(&$db){
		$this->db = $db;
		return false;
	}
	
	/* Get Category data by ID
	*/
	function getCategoryById($id, $get_subcats = true){
		if($id){
			$this->db->query("SELECT *, IF(url_custom='',url_default,url_custom) AS url FROM ".DB_PREFIX."catalog WHERE cid ='".$id."' AND is_visible='Yes'");
			if($this->db->moveNext()){
				$data = $this->db->col;
				$data["url"] = getCatalogUrl($data["url"], $data["cid"]);
				// get subcats
				if($data["list_subcats"] == "Yes"){
					$data["subcategories"] = $this->getSubCategoriesById($id);
				}
				else{
					$data["subcategories"] = false;
				}
				// check image here
				$f = $this->catalog_images_dir."/category".$id.".";
				if(file_exists($f."jpg")){
					$data["img"] = $f."jpg";
				}
				elseif(file_exists($f."png")){
					$data["img"] =  $f."png";
				}
				elseif(file_exists($f."gif")){
					$data["img"] =  $f."gif";
				}
				else{
					$data["img"] = false;
				}
				return $data;
			}
		}
		return false;
	}
	
	/* Return direct subcategories for selected category
	*/
	function getSubCategoriesById($id){
		$this->db->query("SELECT cid FROM ".DB_PREFIX."catalog WHERE parent='".$id."' AND is_visible='Yes' ORDER BY priority, name");
		if($this->db->numRows() > 0){
			$subcats = $this->db->getRecords();
			for($i = 0; $i < count ($subcats); $i++){
				$subcats[$i] = $this->getCategoryById($subcats[$i]["cid"], false);
			}
			return $subcats;
		}
		else{
			return false;
		}
	}
	
	
	
	
	// 2010-07-10 pkolas display number of products in each category
	// this might be bad for performance, but the client wants count next to each category.. so be it.
	function pkUpdateProductCount() {
		$this->db->reset();
		$this->db->query("
			SELECT `cid`
			FROM `".DB_PREFIX."catalog`
			WHERE `parent` = 0
		");
		while ($this->db->movenext()) {
			$this->pkUpdateProductCount2($this->db->col['cid']);
		}
	}
	function pkUpdateProductCount2($cid) {
		if (is_array($this->tree[$cid]["children"])){
			foreach($this->tree[$cid]["children"] as $child_id=>$child_data){
				$this->tree[$cid]["pk_product_ids"] = array_merge($this->tree[$cid]["pk_product_ids"], $this->pkUpdateProductCount2($child_id));
			}
			$this->tree[$cid]['pk_product_ids'] = array_unique($this->tree[$cid]['pk_product_ids']);
		}
		$this->tree[$cid]['pk_num_products'] = count($this->tree[$cid]['pk_product_ids']);
		return $this->tree[$cid]['pk_product_ids'];
	}
	// 2010-07-10 pkolas end
	
	
	
	
	
	
	
	/**
	 * returns parse categories tree
	 *
	 * @param int $category_id
	 * @return array
	 */
	function getParsedTree($category_id=0){
		
		
		// 2010-07-10 pkolas display number of items in each category
		$sql = "
			SELECT `products_categories`.`cid`, `products_categories`.`pid`
			FROM `products_categories` 
				INNER JOIN `products` ON `products_categories`.`pid` = `products`.`pid`
			WHERE `products`.`is_visible` = 'Yes'
		";
		$this->db->reset();
		$this->db->query($sql);
		$pk_product_ids = array();
		while ($this->db->movenext()) {
			if (!isset($pk_product_ids[$this->db->col['cid']])) {
				$pk_product_ids[$this->db->col['cid']] = array();
			}
			$pk_product_ids[$this->db->col['cid']][] = $this->db->col['pid'];
		}
		$this->db->reset();
		// 2010-07-10 pkolas end
		
		
		
		
		$this->tree = array();
		$this->db->query("SELECT cid, parent, level, is_visible, key_name, name, IF(url_custom='',url_default,url_custom) AS url FROM ".DB_PREFIX."catalog ORDER BY level, priority, name");
		if($this->db->numRows()>0){
			while($this->db->moveNext()){
				
				$cid = $this->db->col["cid"];
				$parent = $this->db->col["parent"];
				
				$this->tree[$cid] =	array(
					"cid"=>$cid,
					"parent"=>$parent,
					"level"=>$this->db->col["level"],
					"is_visible"=>$this->db->col["is_visible"],
					"key_name"=>$this->db->col["key_name"],
					"name"=>$this->db->col["name"],
					"children"=>false,
					"category_url"=>getCatalogUrl($this->db->col["url"], $cid)
					
					// 2010-07-10 pkolas display number of items in each category
					, 'pk_product_ids' => (isset($pk_product_ids[$cid]) ? $pk_product_ids[$cid] : array())
					// 2010-07-10 pkolas end
					
				);
				
				if(array_key_exists($parent, $this->tree) && $this->tree[$parent] > 0){
					$this->tree[$parent]["children"][$cid] = &$this->tree[$cid];
				}
			}
			//prepare tree -- levels
			foreach ($this->tree as $id=>$node)
			{
				$level = 0;
				$n = $node;
				$j = 0;
				for ($j = 0; $j<count($this->tree); $j++)
				{
					if ($n["parent"] > 0 && array_key_exists($n["parent"], $this->tree))
					{
						$n = $this->tree[$n["parent"]];
					}
					else
					{
						break;
					}
				}
				$this->tree[$id]["level"] = $j + 1;
				$this->tree[$id]["alevel"] = $this->tree[$id]["level"]>1?array_fill(0, $this->tree[$id]["level"]-1, 1):false;
				$this->tree[$id]["open"] = false;
				
				if ($id == $category_id)
				{
					$_id = $id;
					$this->tree[$_id]["open"] = true;
					for ($i=0; $i<count($this->tree); $i++)
					{
						if ($this->tree[$_id]["parent"] > 0)
						{
							$_id = $this->tree[$_id]["parent"];
							$this->tree[$_id]["open"] = true;
						}
						else
						{
							break;
						}
					}
				}
			}
			
			
			// 2010-07-10 pkolas display product count
			$this->pkUpdateProductCount();
			// 2010-07-10 end
			
			return $this->tree;
		}
		else
		{	
			return false;
		}
	}
	
	/**
	 * Scans all nested children and set to parent category
	 *
	 * @param int $category_id
	 */
	function setCategoryNestedChildrenIds($category_id = 0){
		if(array_key_exists($category_id, $this->tree)){
			$this->tree[$category_id]["nested"] = array($category_id);
			$this->setCategoryNestedChildrenIds2($category_id, $category_id);
		}
	}
	
	/**
	 * Real scan function for setCategoryNestedChildrenIds
	 *
	 * @param int $parent_id
	 * @param int $category_id
	 */
	function setCategoryNestedChildrenIds2($parent_id, $category_id){
		if(is_array($this->tree[$parent_id]["children"])){
			foreach($this->tree[$parent_id]["children"] as $child_id=>$child_data){
				$this->tree[$category_id]["nested"][] = $child_id;
				$this->setCategoryNestedChildrenIds2($child_id, $category_id);
			}
		}
	}
	
	/**
	 * get parsed tree path from category_id node to top
	 *
	 * @param int $category_id
	 * @return array
	 */
	function getParsedPath($category_id){
		if(array_key_exists($category_id, $this->tree)){
			$parsed_path = array();
			$_category_id = $category_id;
			for($i=0; $i<count($this->tree); $i++){
				$node = $this->tree[$_category_id];
				$parsed_path[] = array("href"=>$node["category_url"], "caption"=>$node["name"]);
				if($node["parent"] > 0){
					$_category_id = $node["parent"];
				}
				else{
					break;
				}	
			}
			return array_reverse($parsed_path);
		}
		else{
			return false;
		}
	}
}
?>
