<?php
if(!class_exists("DobaAPI")) require_once("content/classes/class.doba.php");
/**
 * Doba Product API Class
 *
 */
class DobaProductAPI extends DobaAPI 
{
	
	/**
	 * Get suppliers list
	 *
	 * @return mixed
	 */
	public function getSuppliers()
	{
		$r = $this->call("<action>getSuppliers</action>");
		return $r ? $r["dce"][0]["response"][0]["suppliers"] : false;
	}
	
	/**
	 * Get brands
	 *
	 * @return mixed
	 */
	public function getBrands()
	{
		$r = $this->call("<action>getBrands</action>");
		return $r ? $r["dce"][0]["response"][0]["brands"][0]["brand"] : false;
	}
	
	/**
	 * Get quantity options
	 *
	 * @return mixed
	 */
	public function getQtyOptions()
	{
		$r = $this->call("<action>getQtyOptions</action>");
		return $r ? $r["dce"][0]["response"][0]["qty_options"] : false;
	}
	
	/**
	 * Get price ranges
	 *
	 * @return mixed
	 */
	public function getPriceRanges()
	{
		$r = $this->call("<action>getPriceRanges</action>");
		return $r ? $r["dce"][0]["response"][0]["price_ranges"] : false;
	}
	
	/**
	 * Get products by params
	 *
	 * @param array $params
	 * @return mixed
	 */
	public function getProductSearch($params = array())
	{
		return $this->call("<action>getProductSearch</action>");
	}
	
	/**
	 * Return specific product details
	 *
	 * @param mixed $item_id
	 * @return mixed
	 */
	public function getItemDetail($item_id)
	{
		if (is_array($item_id))
		{
			$ids_xml = "";
			foreach ($item_id as $_item_id)
			{
				$ids_xml = $ids_xml."<item>".intval($_item_id)."</item>";
			}
			$r = $this->call("<action>getItemDetail</action><items>".$ids_xml."</items>");
			return $r ? $r["dce"][0]["response"][0]["items"][0]["item"] : false;
		}
		else
		{
			$r = $this->call("<action>getItemDetail</action><items><item>".intval($item_id)."</item></items>");
			return $r ? $r["dce"][0]["response"][0]["items"][0]["item"][0] : false;
		}
	}
	
	/**
	 * Returns invenory data for product
	 *
	 * @param unknown_type $params
	 * @return unknown
	 */
	public function getProductInventory($params = array())
	{
		return $this->call("<action>getProductInventory</action>");
	}
	
	/**
	 * Returns merchants watchlists
	 *
	 * @return mixed
	 */
	public function getWatchlists()
	{
		$r = $this->call("<action>getWatchlists</action>");
		return $r ? isset($r["dce"][0]["response"][0]["watchlists"][0]["watchlist"]) ? $r["dce"][0]["response"][0]["watchlists"][0]["watchlist"] : array() : false;
	}
	
	/**
	 * Returns watchlist products
	 *
	 * @param int $watchlist_id
	 * @return mixed
	 */
	public function getWatchlistProducts($watchlist_id)
	{
		$r = $this->call("<action>getWatchlistProducts</action><watchlist_id>".intval($watchlist_id)."</watchlist_id>");
		return $r ? $r["dce"][0]["response"][0]["ApiGetWatchlistProductsResponse"][0]["products"][0]["watchlist_items"] : false;
	}
	
	/**
	 * Returns watchlist products with additional information
	 *
	 * @param int $watchlist_id
	 * @return mixed
	 */
	public function getWatchlistProductsExtended($watchlist_id)
	{
		if (($products = $this->getWatchlistProducts($watchlist_id)) != false)
		{
			if(count($products > 0))
			{
				$item_id = array();
				foreach ($products as $product)
				{
					$item_id[] = $product["item_id"][0];
				}
				
				$items = $this->getItemDetail($item_id);
				
				return $items;
			}
			
			return $products;
		}
		
		return false;
	}
	
	/**
	 * Update watchlist
	 * @param array $params
	 * @return mixed
	 */
	public function editWatchlist($params = array())
	{
		return $this->call("<action>editWatchlist</action>");
	}
	
	/**
	 * Update item inventory
	 * @param array $item
	 * @param mixed $price
	 */
	public function updateInventory($item, $price)
	{
		//check is product already in database
		$this->db()->query("SELECT * FROM ".DB_PREFIX."products WHERE product_id='".$this->db()->escape($item["item_id"][0])."'");
		$prod = $this->db()->moveNext();
		
		//do not add new products, only update stock / price for existing products
		if($prod)
		{
			//check stock			
			$stock = $item["stock"][0];
			$qty_avail = isset($item["qty_avail"][0]) ? $item["qty_avail"][0] : 0;			
			$in_stock = $stock == "in-stock" && $qty_avail >= $this->min_stock;
			
			$this->db()->reset();
			
			if ($price)
			{
				$this->db()->assignStr("price", round($price, 2));
			}
			$this->db()->assignStr("cost", round($item["price"][0], 2));
			$this->db()->assignStr("inventory_control", "Yes");
			$this->db()->assignStr("inventory_rule", "OutOfStock");
			$this->db()->assignStr("stock", $qty_avail);
			$this->db()->assignStr("is_visible", $in_stock ? "Yes" : "No");
			$this->db()->update(DB_PREFIX."products", "WHERE pid='".intval($prod["pid"])."'");
		}
	}
	
	/**
	 * Inserts/update doba item
	 * @param array $item
	 * @param float $price
	 * @return int 0 - nothing done, 1 - added, 2 - updated
	 */
	public function itemToDb($item, $price = false)
	{	
		$status = 0;
		//check category
		if (isset($item["categories"]) && isset($item["categories"][0]) && isset($item["categories"][0]["category"]) && is_array($item["categories"][0]["category"]))
		{
			//check stock			
			$stock = $item["stock"][0];
			$qty_avail = isset($item["qty_avail"][0]) ? $item["qty_avail"][0] : 0;			
			$in_stock = $stock == "in-stock" && $qty_avail >= $this->min_stock;
			
			//check is product already in database
			$this->db()->query("SELECT * FROM ".DB_PREFIX."products WHERE product_id='".$this->db()->escape($item["item_id"][0])."'");
			$prod = $this->db()->moveNext();
			
			//do not add new products if out of stock
			if($in_stock || $prod)
			{
				$this->db()->reset();
				$this->db()->assignStr("product_id", $item["item_id"][0]);
				
				if (!$prod || ($prod && $prod["is_locked"] != "Yes"))
				{
					$this->db()->assignStr("title", isset($item["item_name"]) && isset($item["item_name"][0]) && $item["item_name"][0] != $item["product_title"][0] ? ($item["product_title"][0]." - ".$item["item_name"][0]) : $item["product_title"][0]);
					$this->db()->assignStr("description", isset($item["description"]) && isset($item["description"][0]) ? $item["description"][0] : "");
				}
				$this->db()->assignStr("product_sku", isset($item["sku"]) && isset($item["sku"][0]) ? $item["sku"][0] : "");
				$this->db()->assignStr("product_upc", isset($item["upc"]) && isset($item["upc"][0]) ? $item["upc"][0] : "");
				$this->db()->assignStr("weight", isset($item["ship_weight"]) && isset($item["ship_weight"][0]) ? $item["ship_weight"][0] : "0.00");
				$this->db()->assignStr("dimension_width", isset($item["ship_width"]) && isset($item["ship_width"][0]) ? $item["ship_width"][0] : "0.00");
				$this->db()->assignStr("dimension_height", isset($item["ship_height"]) && isset($item["ship_height"][0]) ? $item["ship_height"][0] : "0.00");
				$this->db()->assignStr("dimension_length", isset($item["ship_length"]) && isset($item["ship_length"][0]) ? $item["ship_length"][0] : "0.00");

				if ($price)
				{
					$this->db()->assignStr("price", round($price, 2));
				}
				$this->db()->assignStr("cost", round($item["price"][0], 2));
			
				$this->db()->assignStr("inventory_control", "Yes");
				$this->db()->assignStr("inventory_rule", "OutOfStock");
				$this->db()->assignStr("stock", $qty_avail);
				$this->db()->assignStr("is_doba", "Yes");
				$this->db()->assignStr("is_visible", $in_stock ? "Yes" : "No");
				
				//images
				$image_url = "";
				if (isset($item["images"]) && isset($item["images"][0]) && isset($item["images"][0]["image"]) && isset($item["images"][0]["image"][0]))
				{
					$image_default = $item["images"][0]["image"][0]["url"][0];
					foreach ($item["images"][0]["image"] as $image)
					{
						if(isset($image["default"]) && isset($image["default"][0]) && $image["default"][0] == 1)
						{
							$image_url = $image["url"][0];
						}
					}
				}
			
				if ($image_url != "")
				{
					$this->db()->assignStr("image_location", "Web");
					$this->db()->assignStr("image_url", $image_url);
				}
				
				//add or update product
				if ($prod)
				{
					//product exists, do update
					$product_id_in_db = $prod["pid"];
					$this->db()->update(DB_PREFIX."products", "WHERE pid='".$product_id_in_db."'");
					$status = 1;
				}
				else
				{
					//new product, insert
					$this->db()->assign("added", "NOW()");
					$product_id_in_db = $this->db()->insert(DB_PREFIX."products");
					$status = 2;
				}
			
				//check category
				$level = 1;
				$parent = 0;
				//we have to skip very first level as it is not real category
				for ($key=0; $key < count($item["categories"][0]["category"]); $key++)
				{
					$category = $item["categories"][0]["category"][$key];
					$this->db()->query("SELECT * FROM ".DB_PREFIX."catalog WHERE key_name = '".$this->db()->escape("doba_".$category["id"][0])."'");
					if (($cat = $this->db()->moveNext()) !=  false)
					{
						$parent = $cat["cid"];
					}
					else
					{
						$this->db()->reset();
						$this->db()->assignStr("parent", $parent);
						$this->db()->assignStr("level", $level);
						$this->db()->assignStr("priority", 5);
						$this->db()->assignStr("is_visible", "Yes");
						$this->db()->assignStr("list_subcats", "No");
						$this->db()->assignStr("key_name", "doba_".$category["id"][0]);
						$this->db()->assignStr("name", ucfirst($category["name"][0]));
						$parent = $this->db()->insert(DB_PREFIX."catalog");
					}
					
					$level++;
				}
				
				//update category
				$this->db()->reset();
				$this->db()->assign("cid", $parent);
				$this->db()->update(DB_PREFIX."products", "WHERE pid='".$product_id_in_db."'");
				
				$this->db()->query("DELETE FROM ".DB_PREFIX."products_categories WHERE pid='".$product_id_in_db."'");
				
				$this->db()->reset();
				$this->db()->assign("cid", $parent);
				$this->db()->assign("pid", $product_id_in_db);
				$this->db()->assign("is_primary", 1);
				$this->db()->insert(DB_PREFIX."products_categories");
			}
		}
		return $status;
	}
}