<?php

	define("PAYMENT_TRANSACTION_ERROR_TEXT", "We were unable to complete your transaction.  Please check your information to ensure it is correct, or try another card. ");
	define("PAYMENT_SERVER_ERROR_TEXT", "We are sorry, but an error occurred while processing your transaction. Please try again or contact the site administrator. ");
	
	$payment_types_def = array(
		"cc"=>"Credit Card",
		"check"=>"Check"
	);
	
	///////////////////////////////////////////////////
	//
	class PAYMENT_METHOD
	{
		var $pid;
		var $protocol;
		var $type;
		var $url_to_gateway = "";
		var $name;
		var $title;
		var $message_payment;
		var $message_thankyou;
		var $id;
		var $class_name;
		
		function PAYMENT_METHOD($pid, $protocol, $type, $url_to_gateway, $name, $title, $message_payment, $message_thankyou, $id, $class_name)
		{
			$this->pid = $pid;
			$this->protocol = $protocol;
			$this->type = $type;
			$this->url_to_gateway = $url_to_gateway;
			$this->name = $name;
			$this->title = $title;
			$this->message_payment = $message_payment;
			$this->message_thankyou = $message_thankyou;
			$this->id = $id; // used for payment gateways
			$this->class_name = $class_name;
			return $this;
		}
	}

	/**
	 * Enter description here...
	 *
	 */
	class PAYMENT{
		var $methods = array();
		var $is_error = false;
		var $error_message;
		var $is_ipn_method = false;
		
		function PAYMENT(&$db)
		{
			global $payment_types_def;
			global $settings;

			$db->query("
				SELECT * FROM ".DB_PREFIX."payment_methods
				WHERE ".DB_PREFIX."payment_methods.active='Yes'
				ORDER BY ".DB_PREFIX."payment_methods.priority, name
			");

			$this->is_ipn_method = false;
			while ($db->moveNext())
			{
				$may_be_added = true;
				if(isset($_SESSION["paypal_express_checkout_details"]) && $db->col["id"] != "paypalec" && $settings["paypalec_Exclusive"] == "Yes"){
					$may_be_added = false;
				}
				if($db->col["id"] == "paypalec" && !isset($_SESSION["paypal_express_checkout_details"])){
					$may_be_added = false;
				}
				if($db->col["id"] == "googlecheckout" ){
					$may_be_added = false;
				}

				if ($may_be_added)
				{
					$this->methods[$db->col["pid"]] = new PAYMENT_METHOD(
						$db->col["pid"],
						$db->col["proto"],
						$db->col["type"],
						$db->col["url_to_gateway"],
						$db->col["name"],
						$db->col["title"],
						$db->col["message_payment"],
						$db->col["message_thankyou"],
						$db->col["id"],
						$db->col["class"]
					);
					
					if ($db->col["type"] == "ipn")
					{
						$this->is_ipn_method = true;
					}
				}
			}
			return $this;
		}
		
		function isMethod($type){
			return array_key_exists($type, $this->methods);
		}
		
		function isMethods(){
			return count($this->methods);
		}
	}
	
	////////////////////////////////////////
	// Base class for all payment processors (gateways)
	class PAYMENT_PROCESSOR
	{
		var $id;
		var $name;
		var $version;
		var $class_name;
		var $type;
		var $description;
		var $testMode = true;
		
		var $post_url = "";
		
		var $common_vars;
		var $settings_vars;
		var $custom_vars;
		
		var $is_error = false;
		var $error_message = "";
		
		var $user = false;
		var $order = false;
		var $order_items = false;
		
		var $show_cvv2 = true;
		var $is_certificate = false;
		var $certificate_path = "";
		
		var $support_accepted_cc = true;
		var $requires_cc_start_date = false;
		var $requires_issue_number = false;
		var $has_issue_number = false;
		var $need_cc_codes = false;
		var $payment_page = "invoice";
		
		var $possible_payment_delay = false;
		var $possible_payment_delay_timeout = 0; // seconds
		
		var $db_id = 0;
		
		private $_logFile = false;
		public $logFileName = "";
		public $enableLog = false;
		
		var $support_ccs = false;
		var $enable_ccs = false;
		
		var $currency = false;
		
		function assignUser(&$user){
			$this->user = $user;
		}
		
		function assignOrder(&$order){
			$this->order = $order;
		}
		
		function assignOrderItems(&$order_items){
			$this->order_items = $order_items;
		}
		
		function getSettingsData(&$db){
			global $settings;
			$db->query("SELECT * FROM ".DB_PREFIX."payment_methods WHERE pid='".$this->db_id."'");
			if($db->moveNext()){
				$this->url_to_gateway = $db->col["url_to_gateway"];
				$this->enable_ccs = 
					$db->col["enable_ccs"] == 1 && 
					$settings["SecurityCCSActive"] == "1" && 
					$settings["SecurityCCSCertificate"] !=  "" && 
					$settings["SecurityCCSPrivateKey"] != "";
			}
			else{
				$db->query("SELECT * FROM ".DB_PREFIX."payment_methods WHERE id='".($this->id)."'");
				if($db->moveNext()){
					$this->url_to_gateway = $db->col["url_to_gateway"];
					$this->enable_ccs = 
						$db->col["enable_ccs"] == 1 && 
						$settings["SecurityCCSActive"] == "1" && 
						$settings["SecurityCCSCertificate"] !=  "" && 
						$settings["SecurityCCSPrivateKey"] != "";
				}
			}
			$db->query("SELECT * FROM ".DB_PREFIX."settings WHERE group_name LIKE 'payment_".($this->id)."'");
			while($db->moveNext()){
				$this->settings_vars[$db->col["name"]] = array(
					"name" => $db->col["name"],
					"value" => $db->col["value"],
					"description" => $db->col["name"],
					"input_type" => "hidden"
				);
			}
		}
		
		function getCreditCardOptions(&$db){
			$db->query("SELECT * FROM ".DB_PREFIX."payment_accepted_cc WHERE pid='".$this->db_id."' AND enable='Yes'");
			if($db->numRows()>0){
				$options = "";
				while($db->moveNext()){
					$options.='<option value="'.($db->col["enable_cvv2"]).'">'.gs($db->col["name"]).'</option>';
				}
				return $options;
			}
			return false;
		}
		
		function getCreditCardOptionsCodes(&$db){
			$db->query("SELECT * FROM ".DB_PREFIX."payment_accepted_cc WHERE pid='".$this->db_id."' AND enable='Yes'");
			if($db->numRows()>0){
				$options = "";
				while($db->moveNext()){
					$options.='<option value="'.($db->col["code"]).'">'.gs($db->col["name"]).'</option>';
				}
				return $options;
			}
			return false;
		}
		
		function addCommon($name){
			$this->common_vars[$name] = array(
				"name"=>"",
				"value"=>"",
				"description"=>"",
				"input_type"=>"hidden"
			);
		}
		
		function setCommonValue($name, $value){
			$this->common_vars[$name]["value"] = $value;
		}
		
		function assignCommon($name, $rename, $value, $description, $input_type){
			$this->common_vars[$name] = array(
				"name"=>$rename,
				"value"=>$value,
				"description"=>$description,
				"input_type"=>$input_type
			);
		}
		
		function assignCustom($name, $value, $description, $input_type){
			$this->custom_vars[$name] = array(
				"name"=>$name,
				"value"=>$value,
				"description"=>$description,
				"input_type"=>$input_type
			);
		}
		
		function uninstall(&$db){
			$db->query("DELETE FROM ".DB_PREFIX."payment_methods WHERE id='".$this->id."'");
			$db->query("DELETE FROM ".DB_PREFIX."settings WHERE group_name='payment_".$this->id."'");
			@unlink("content/engine/payment/payment_".$this->id.".php");
		}
		
		function PAYMENT_PROCESSOR(){
			global $_SERVER, $_SESSION;
			$this->common_vars = array();
			$this->custom_vars = array();
			$this->settings_vars = array();
			$this->post_url = "";
			$this->protocol = "";
			
			// register common vars
			$this->addCommon("customer_id");
			
			$this->addCommon("billing_first_name");
			$this->addCommon("billing_last_name");
			$this->addCommon("billing_name");
			$this->addCommon("billing_company");
			$this->addCommon("billing_address");
			$this->addCommon("billing_address1");
			$this->addCommon("billing_address2");
			$this->addCommon("billing_city");
			$this->addCommon("billing_state");
			// wmw 03/30/05 added for paypal ipn
			$this->addCommon("billing_state_abbr");
			
			$this->addCommon("billing_country");
			$this->addCommon("billing_country_iso_a2");
			$this->addCommon("billing_country_iso_a3");
			$this->addCommon("billing_country_iso_number");
			$this->addCommon("billing_zip");
			$this->addCommon("billing_phone");
			$this->addCommon("billing_email");
			
			$this->addCommon("shipping_digital");
			$this->addCommon("shipping_name");
			$this->addCommon("shipping_company");
			$this->addCommon("shipping_address");
			$this->addCommon("shipping_address1");
			$this->addCommon("shipping_address2");
			$this->addCommon("shipping_city");
			$this->addCommon("shipping_state");
			// wmw 03/30/05 added for paypal ipn
			$this->addCommon("shipping_state_abbr");
			$this->addCommon("shipping_country");
			$this->addCommon("shipping_country_iso_a2");
			$this->addCommon("shipping_country_iso_a3");
			$this->addCommon("shipping_country_iso_number");
			$this->addCommon("shipping_zip");
			
			$this->addCommon("order_id");
			$this->addCommon("order_total_amount");
			$this->addCommon("order_security_id");
			$this->addCommon("order_tax_amount");
			$this->addCommon("order_shipping_amount");
			
			$this->currency = $_SESSION["default_currency"];
			
			$this->logFileName = "content/cache/log/payment-".$this->id.".txt";
			$this->logStart();
			return $this;
		}
		
		function getCommonData(&$user, &$order){
			$this->setCommonValue("customer_id", $user->id);
			$this->setCommonValue("billing_name", $user->data["fname"]." ".$user->data["lname"]);
			$this->setCommonValue("billing_last_name", $user->data["lname"]);
			$this->setCommonValue("billing_first_name", $user->data["fname"]);
			$this->setCommonValue("billing_company", $user->data["company"]);
			$this->setCommonValue("billing_address", trim($user->data["address1"].(trim($user->data["address2"])!=""?("; ".$user->data["address2"]) : "")));
			$this->setCommonValue("billing_address1", $user->data["address1"]);
			$this->setCommonValue("billing_address2", $user->data["address2"]);
			$this->setCommonValue("billing_city", $user->data["city"]);
			$this->setCommonValue("billing_state", $user->data["province"]);
			$this->setCommonValue("billing_state_abbr", $user->data["state_abbr"]);
			$this->setCommonValue("billing_country", $user->data["country_name"]);
			$this->setCommonValue("billing_country_iso_a2", $user->data["country_iso_a2"]);
			$this->setCommonValue("billing_country_iso_a3", $user->data["country_iso_a3"]);
			$this->setCommonValue("billing_country_iso_number", $user->data["country_iso_number"]);
			$this->setCommonValue("billing_zip", $user->data["zip"]);
			$this->setCommonValue("billing_phone", $user->data["phone"]);
			$this->setCommonValue("billing_email", $user->data["email"]);
			
			$this->setCommonValue("shipping_digital", $order->shippingDigital);
			$this->setCommonvalue("shipping_name", $order->shippingAddress["name"]);
			$this->setCommonvalue("shipping_company", $order->shippingAddress["company"]);
			$this->setCommonvalue("shipping_address", trim($order->shippingAddress["address1"].(trim($order->shippingAddress["address2"])!=""?("; ".$order->shippingAddress["address2"]) : "")));
			$this->setCommonvalue("shipping_address1", $order->shippingAddress["address1"]);
			$this->setCommonvalue("shipping_address2", $order->shippingAddress["address2"]);
			$this->setCommonvalue("shipping_city", $order->shippingAddress["city"]);
			$this->setCommonvalue("shipping_state", $order->shippingAddress["province"]);
			$this->setCommonvalue("shipping_state_abbr", $order->shippingAddress["state_abbr"]);
			$this->setCommonvalue("shipping_zip", $order->shippingAddress["zip"]);
			$this->setCommonvalue("shipping_country", $order->shippingAddress["country"]);
			$this->setCommonValue("shipping_country_iso_a2", $order->shippingAddress["country_iso_a2"]);
			$this->setCommonValue("shipping_country_iso_a3", $order->shippingAddress["country_iso_a3"]);
			$this->setCommonValue("shipping_country_iso_number", $order->shippingAddress["country_iso_number"]);
			
			$this->setCommonValue("order_id", $order->order_num);
			$this->setCommonValue("order_oid", $order->oid);
			// *** GIFT CERTIFICATE START
			$this->setCommonValue("order_total_amount", $order->totalAmount-$order->gift_cert_amount);
			// *** GIFT CERTIFICATE END
			$this->setCommonValue("order_security_id", md5($order->oid."-".$order->order_num));
			$this->setCommonValue("order_tax_amount", $order->taxAmount);
			$this->setCommonValue("order_subtotal_amount", $order->subtotalAmount);
			$this->setCommonValue("order_shipping_amount", $order->shippingAmount);
			$this->setCommonValue("order_handling_amount", $order->handlingAmount);
			$this->setCommonValue("order_handling_separated", $order->handlingSeparated);
			$this->setCommonValue("order_discount_amount", $order->discountAmount);
		}
		
		function getMonthOptions(){
			return 
				"<option value=\"01\">January</option>\r\n".
				"<option value=\"02\">February</option>\r\n".
				"<option value=\"03\">March</option>\r\n".
				"<option value=\"04\">April</option>\r\n".
				"<option value=\"05\">May</option>\r\n".
				"<option value=\"06\">June</option>\r\n".
				"<option value=\"07\">July</option>\r\n".
				"<option value=\"08\">August</option>\r\n".
				"<option value=\"09\">September</option>\r\n".
				"<option value=\"10\">October</option>\r\n".
				"<option value=\"11\">November</option>\r\n".
				"<option value=\"12\">December</option>\r\n";
		}
		
		function getYearOptions(){
			$start = date("Y");
			$fin = $start + 21;
			$o = "";
			for($i=$start; $i<$fin; $i++){
				$o.="<option value=\"".$i."\">".$i."</option>\r\n";
			}
			return $o;
		}
		
		function getPaymentForm($db){
			global $msg;
			global $_SESSION;
			$this->cc_options = $this->getCreditCardOptions($db);
			$this->cc_options_codes = $this->getCreditCardOptionsCodes($db);
			
			if($this->possible_payment_delay){
				$_t_o = isset($_SESSION["payment_session_delay_timeout"]) ? $_SESSION["payment_session_delay_timeout"] : 0;
				$_SESSION["payment_session_delay_timeout"] = $_t_o > $this->possible_payment_delay_timeout ? $_t_o : $this->possible_payment_delay_timeout;
			}
			
			$fields = array();
	
			if($this->type == "cc"){
				
				$accepted_cc = array();
				
				$db->query("SELECT * FROM ".DB_PREFIX."payment_accepted_cc WHERE pid='".$this->db_id."' AND enable='Yes'");
				if($db->numRows()>0){
					while($db->moveNext()){
						$accepted_cc[] = array("value"=>$this->need_cc_codes?$db->col["code"]:$db->col["enable_cvv2"], "caption"=>$db->col["name"]);
					}
				}
				
				$cc_start_years = array();
				for($i=date("Y")-10; $i<=date("Y")+10; $i++){
					$cc_start_years[] = array("value"=>$i, "caption"=>$i);
				}
				
				$cc_exp_years = array();
				for($i=date("Y"); $i<=date("Y")+10; $i++){
					$cc_exp_years[] = array("value"=>$i, "caption"=>$i);
				}
				
				$cc_start_months = array();
				for($i=1; $i<=12; $i++){
					$cc_start_months[] = array("value"=>($i<10?"0":"").$i, "caption"=>$i);
				}
				
				$cc_exp_months = array();
				for($i=1; $i<=12; $i++){
					$cc_exp_months[] = array("value"=>($i<10?"0":"").$i, "caption"=>$i);
				}
				
				$fields["oa"] = array("type"=>"hidden", "name"=>"oa", "value"=>ORDER_PROCESS_PAYMENT);
				$fields["cc_first_name"] = array("type"=>"text", "name"=>"form[cc_first_name]", "value"=>$this->common_vars["billing_first_name"]["value"], "caption"=>$msg["billing"]["cc_first_name"]);
				$fields["cc_last_name"] = array("type"=>"text", "name"=>"form[cc_last_name]", "value"=>$this->common_vars["billing_last_name"]["value"], "caption"=>$msg["billing"]["cc_last_name"]);
				
				if(count($accepted_cc) > 0){
					$fields["cc_type"] = array("type"=>"select", "name"=>"form[cc_type]", "value"=>"", 
						"options"=>$accepted_cc, "caption"=>$msg["billing"]["cc_type"]);
				}
				$fields["cc_number"] = array("type"=>"text", "name"=>"form[cc_number]", "value"=>"", "caption"=>$msg["billing"]["cc_number"]);
				
				if(isset($this->requires_cc_start_date)){
					if($this->requires_cc_start_date){
						$fields["cc_start_year"] = array("type"=>"select", "name"=>"form[cc_start_year]", "value"=>date("Y"), "options"=>$cc_start_years, "caption"=>(isset($msg["billing"]["cc_start_date"])?$msg["billing"]["cc_start_date"]:"Credit Card Start Date"), "hint"=>(isset($msg["billing"]["cc_start_date_hint"])?$msg["billing"]["cc_start_date_hint"]:"Required for Switch and Solo cards"));
						$fields["cc_start_month"] = array("type"=>"select", "name"=>"form[cc_start_month]", "value"=>date("n"), "options"=>$cc_start_months, "caption"=>(isset($msg["billing"]["cc_start_date"])?$msg["billing"]["cc_start_date"]:"Credit Card Start Date"), "hint"=>(isset($msg["billing"]["cc_start_date_hint"])?$msg["billing"]["cc_start_date_hint"]:"Required for Switch and Solo cards"));
					}
				}
				
				if($this->has_issue_number || isset($this->requires_issue_number)){
					if($this->has_issue_number || $this->requires_issue_number){
						$fields["cc_issue_number"] = array("type"=>"text", "name"=>"form[cc_issue_number]", "value"=>"", "caption"=>$msg["billing"]["card_issue_number"]);
					}
				}
				$fields["cc_expiration_year"] = array("type"=>"select", "name"=>"form[cc_expiration_year]", "value"=>date("Y"), "options"=>$cc_exp_years, "caption"=>$msg["billing"]["cc_expiration_date"]);
				$fields["cc_expiration_month"] = array("type"=>"select", "name"=>"form[cc_expiration_month]", "value"=>date("n"), "options"=>$cc_exp_months, "caption"=>$msg["billing"]["cc_expiration_date"]);

				$fields["cc_cvv2"] = array("type"=>"text", "name"=>"form[cc_cvv2]", "maxlength"=>"4", "size"=>"4", "value"=>"", "caption"=>$msg["billing"]["cc_cvv2"], "hint"=>$msg["billing"]["cc_cvv2_hint"]);
			}
			
			if($this->type == "check"){
				$fields["oa"] = array("type"=>"hidden", "name"=>"oa", "value"=>ORDER_PROCESS_PAYMENT);
				$fields["check_bank_aba_code"] = array("type"=>"text", "name"=>"form[check_bank_aba_code]", "value"=>"", "caption"=>$msg["billing"]["check_bank_aba_code"]);
				$fields["check_bank_account_number"] = array("type"=>"text", "name"=>"form[check_bank_account_number]", "value"=>"", "caption"=>$msg["billing"]["check_bank_account_number"]);
				$fields["check_bank_account_type"] = array(
					"type"=>"select", "name"=>"form[check_bank_account_type]", "value"=>"", 
					"options"=>array(
						array("value"=>"checking", "caption"=>$msg["billing"]["check_bank_account_type_checking"]),
						array("value"=>"business_checking", "caption"=>$msg["billing"]["check_bank_account_type_business_checking"]),
						array("value"=>"savings", "caption"=>$msg["billing"]["check_bank_account_type_savings"])
						
					),
					"caption"=>$msg["billing"]["check_bank_account_type"]
				);
				$fields["check_bank_name"] = array("type"=>"text", "name"=>"form[check_bank_name]", "value"=>"", "caption"=>$msg["billing"]["check_bank_name"]);
				$fields["check_bank_account_name"] = array("type"=>"text", "name"=>"form[check_bank_account_name]", "value"=>$this->common_vars["billing_name"]["value"], "caption"=>$msg["billing"]["check_bank_account_name"]);
			}
			
			return $fields;
		}
		
		function getValidatorJS(){
			global $msg;
			if($this->type == "cc"){
				$JSmsg ="\n".
					"if(!validateName(frm.elements['form[cc_first_name]'].value)){\n".
					"	alert('".$msg["billing"]["enter_valid_cc_first_name"]."');\n".
					"	frm.elements['form[cc_first_name]'].focus();\n".
					"	return false;\n".
					"}\n".
					"if(!validateName(frm.elements['form[cc_last_name]'].value)){\n".
					"	alert('".$msg["billing"]["enter_valid_cc_last_name"]."');\n".
					"	frm.elements['form[cc_last_name]'].focus();\n".
					"	return false;\n".
					"}\n".
					"if(!LuhnCheck(frm.elements['form[cc_number]'].value) || !validateCCNum(frm.elements['form[cc_number]'].value)){\n".
					"	alert('".$msg["billing"]["enter_valid_cc_number"]."');\n".
					"	frm.elements['form[cc_number]'].focus();\n".
					"	return false;\n".
					"}\n".
					"if(isCardExpired(frm.elements['form[cc_expiration_month]'].value, frm.elements['form[cc_expiration_year]'].value))
					{
						alert('".addslashes($msg["billing"]["enter_valid_cc_expiration"])."');
						return false;
					}\n".
					"if(!validateCVC2(frm.elements['form[cc_cvv2]'].value)){\n".
					"	alert('".$msg["billing"]["enter_valid_cc_cvv2"]."');\n".
					"	frm.elements['form[cc_cvv2]'].focus();\n".
					"	return false;\n".
					"}\n".
					"if(frm.elements['form[cc_type]'] && (frm.elements['form[cc_type]'].value == 'Yes') && (frm.elements['form[cc_cvv2]'].value == '')){\n".
					"	alert('".$msg["billing"]["enter_valid_cc_cvv2_when_required"]."');\n".
					"	frm.elements['form[cc_cvv2]'].focus();\n".
					"	return false;\n".
					"}\n";
					if($this->requires_issue_number){
						$JSmsg .= "if(!validateText(frm.elements['form[cc_issue_number]'].value)){\n".
							"	alert('".$msg["billing"]["enter_issue_number"]."');\n".
							"	frm.elements['form[cc_issue_number]'].focus();\n".
							"	return false;\n".
							"}\n";
					}
					return $JSmsg;
			}
			
			if($this->type == "check"){
				return 
					"\n".
					"if(!validateText(frm.elements['form[check_bank_aba_code]'].value)){\n".
					"	alert('".$msg["billing"]["enter_check_bank_aba_code"]."');\n".
					"	frm.elements['form[check_bank_aba_code]'].focus();\n".
					"	return false;\n".
					"}\n".
					"if(!validateText(frm.elements['form[check_bank_account_number]'].value)){\n".
					"	alert('".$msg["billing"]["enter_check_bank_account_number"]."');\n".
					"	frm.elements['form[check_bank_account_number]'].focus();\n".
					"	return false;\n".
					"}\n".
					"if(!validateText(frm.elements['form[check_bank_name]'].value)){\n".
					"	alert('".$msg["billing"]["enter_check_bank_name"]."');\n".
					"	frm.elements['form[check_bank_name]'].focus();\n".
					"	return false;\n".
					"}\n".
					"if(!validateText(frm.elements['form[check_bank_account_name]'].value)){\n".
					"	alert('".$msg["billing"]["enter_check_bank_account_name"]."');\n".
					"	frm.elements['form[check_bank_account_name]'].focus();\n".
					"	return false;\n".
					"}\n";
			}
		}
		
		function processCustom($db, $custom_id, $order_id){
			return false;
		}
		
		function getPostUrl(){
			return $this->post_url;
		}
		
		function getVarStatus($custom1="", $custom2="", $custom3="", $order_status="", $payment_status=""){
			return array("1"=>false, "2"=>false, "3"=>false);
		}
		
		function saveCCdata($db, $order, $card_data, $csr_pem){
			global $settings;
			if(!class_exists("openSSL")){
				require_once("content/classes/class.openssl.php");
			}
			if($csr_pem != ""){
				$openSSL = new openSSL(4096, OPENSSL_KEYTYPE_RSA, 3650, $settings["GlobalServerPath"]."/content/ssl/openssl.cnf");
				$crypted = $openSSL->crypt(serialize($card_data), $csr_pem);
				$data = urlencode(
					serialize(
						array(
							"l4"=>substr($card_data["cc"], strlen($card_data["cc"]) - 4, 4),
							"cr"=>$crypted
						)
					)
				);
				$db->reset();
				$db->assignStr("card_data", $data);
				$db->update(DB_PREFIX."orders", "WHERE oid='".$order->oid."'");
			}
		}
		
		function createTransaction(&$db, &$user, &$order, $response, $extra, $custom1="", $custom2="", $custom3="", $security_id=""){
			$db->reset();
			$db->assignStr("security_id", $security_id);
			$db->assignStr("custom1", $custom1);
			$db->assignStr("custom2", $custom2);
			$db->assignStr("custom3", $custom3);
			$db->update(DB_PREFIX."orders", "WHERE oid='".$order->oid."'");
			
			
			$db->reset();
			$db->assign("oid", $order->oid);
			$db->assign("uid", $user->id);
			
			$db->assign("completed", "NOW()");
			$db->assignStr("extra", $extra);
			
			$db->assignStr("payment_type", $this->type);
			$db->assignStr("payment_gateway", $this->id." (".$this->name.")");
			$db->assignStr("payment_response", $response); 
			
			$db->assignStr("order_subtotal_amount", $order->subtotalAmount);
			// *** GIFT CERTIFICATE START
			$db->assignStr("order_total_amount", $order->totalAmount-$order->gift_cert_amount);
			// *** GIFT CERTIFICATE END
			
			$db->assignStr("shipping_method", "");
			$db->assignStr("shipping_submethod", "");
			$db->assignStr("shipping_amount", $order->shippingAmount);
			
			$db->assignStr("tax_amount", $order->taxAmount);
			
			$db->insert(DB_PREFIX."payment_transactions");
		}
		
		/**
		 * Starts log process by opening file for writing
		 */
		public function logStart()
		{
			if ($this->enableLog)
			{
				$this->logFileName = "content/cache/log/payment-".$this->id.".txt";
				if (!$this->logFile)
				{
					$this->logFile = @fopen($this->logFileName, "a+");
				}
			}
		}
		
		/**
		 * Closes log file
		 */
		public function logFinish()
		{
			if ($this->enableLog)
			{
				if ($this->logFile)
				{
					@fclose($this->logFile);
					$this->logFile = false;
				}
			}
		}	
		
		/**
		 * Writer message into log
		 * @return unknown_type
		 */
		public function log($s)
		{
			if ($this->enableLog)
			{
				if (!$this->logFile) $this->logStart();
				
				@fputs($this->logFile, date("Y/m/d-H:i:s")." - ".$this->id."\n".$s."\n\n");
				@fputs($this->logFile, "--------------------------------------------------\n\n");
			}
		}
		
	}

