<?php
/*	DataCash PHP API
	This module is used to build an XML document

	$Id: DataCash_CardInfo.php,v 1.2.2.2 2006/06/05 16:56:30 victor Exp $
	
	Usage:
		$doc = new DataCash_CardInfo (...)

	Author:	Dave MacRae

	Copyright: DataCash Group plc 2003
	Changelog:
		dem: 2002-05-22 Corrected stupid mistake in _get_data
		dem: 2002-05-22 Corrected get function parameters
		dem: 2002-05-22 Added new Luhn function written in PHP
		dem: 2002-05-23 Added CVS ID tag and class logger

*/

$CVSid = explode (" ", "$Id: DataCash_CardInfo.php,v 1.2.2.2 2006/06/05 16:56:30 victor Exp $");
$included_classes{basename($CVSid{1}, ".php,v")} = $CVSid{2};

Class DataCash_CardInfo {
	
  function DataCash_CardInfo ($params) {

    foreach ($params as $key => $value) {
      $this->{$key}=$value;
    }

    // We assume that the config parameter is an already built XML
    // document.
    // $self->{config}=new DataCash::Config($params{configfile});

    if ($this->{datadir}) {
      $this->{fh_bins_path} = $this->{datadir} . "/CardInfo1.bin";
      $this->{fh_data_path} = $this->{datadir} . "/CardInfo2.bin";
    }

    if (file_exists($this->{fh_bins_path})) {
      $this->{fh_bins} = fopen ($this->{fh_bins_path}, "rb") or die ("Cannot open CardInfo1.bin: $!");
    }

    if (file_exists($this->{fh_data_path})) {
      $this->{fh_data} = fopen($this->{fh_data_path},"rb") or die ("Cannot open CardInfo2.bin: $!");
    }

    // If a PAN has been entered then check it's requirements
    if ($this->{pan}) {
      $this->_find(0);
    }

    return;

  }

  function _unbin ($buffer) {

    $len = strlen($buffer);

    if ($len != 16) {
      dc_log (5, "CardInfo: card not found\n");
      return false;
    }

    $start = $this->_chunk ($buffer, 0, 3);
    $length = $this->_chunk ($buffer,3,1);
    $left = $this->_chunk ($buffer,4,4);
    $right = $this->_chunk ($buffer,8,4);
    $data = $this->_chunk ($buffer,12,4);

    return array ($start,$length,$left,$right,$data);
  }

  function _chunk ($data, $offset, $length) {

    // build the unpack string.
    $template = $this->_x("x", $offset, "nul") . $this->_x("C", $length, "char");
    $octets = unpack($template, $data);

    $i=0;
    $total=0;

    foreach (array_reverse ($octets) as $element) {
      $total += $element * pow(2, $i);
      $i += 8;
    }

    return $total;
  }

  function _x ($char, $count, $name) {
    // This function emulates the Perl "x" oprator.	It has an extra paramerter that is
    // used to build a proper unpack operator. This is not portable back to Perl
    static $index = 1;
    $x = '';
    for ($i = 0;$i < $count;$i++)
      $x .= $char . $name . $index++ . "/";
    return ($x);
  }

  function _find ($offset) {
    $bin = substr($this->{pan}, 0, 6);
    fseek ($this->{fh_bins}, $offset*16, 0);
    $buffer = fread ($this->{fh_bins}, 16);
    list ($start,$length,$left,$right,$data) = $this->_unbin($buffer);
    if (! $start) {
      return;
    }

    if ($bin >= ($start+$length)) {
      $this->_find($right+$offset);
    } else {
      if ($bin>=$start) {
	$this->_data($data);
      } else {
	$this->_find($left+$offset);
      }
    }
  }

  function _get_data ($fh, $offset) {

    rewind($fh);
    while (!feof($fh)) {
      // get more data into the buffer
      $buffer = fread ($fh, 8192);
      $no_newlines = substr_count ($buffer, "\n");
      $run_tot += $no_newlines;
      if ($offset - $no_newlines <= 0) {
	// The data probably appears in this block, but all the data might not be there
	// If the last character isn't a newline then it probably isn't
	$line_array = explode ("\n", $buffer);
	if ($buffer{strlen($buffer) - 1} == "\n") {
	  return ($line_array[$offset]);
	} else {
	  // get a small chunk rom the file
	  $next = fread ($fh, 512);
	  // add it to the existing chunk
	  $last = $buffer . $next;
	  $line_array = explode ("\n", $last);
	  return ($line_array[$offset]);
	}
	// line requires is in this block;
	$line_array = explode ("\n", $buffer);
	return ($line_array[$offset]);
      } else {
	$offset -= $no_newlines;
      }
    }
    return "";
  }

  function _data ($offset) {

# Note the fact that we have located the bin details
    $this->{binfound} = 'true';

    $buffer = $this->_get_data($this->{fh_data}, $offset - 1);
    list ($scheme,$issuer,$country,$requirements) = explode(chr(0), $buffer);
    $this->{scheme}=$scheme;
    $this->{issuer}=$issuer;
    $this->{country}=$country;
    $this->{requirements}=$requirements;

    dc_log(2, "Requirements " . $requirements . "| Scheme $scheme | Issuer $issuer | Country $country ");

    return $this;
  }

  function scheme () {
    return $this->{scheme};
  }

  function issuer () {
    return $this->{issuer};
  }

  function digits () {
    return($this->{requirements} & 31);
  }

  function country () {
    return $this->{country};
  }

  function validnumber () {
    return $this->_luhn();
  }

  function issuenumber () {
    return (($this->{requirements} & 192)/64);
  }

  function startdate () {
    return (($this->{requirements} & 32)!=0);
  }

  function cpass () {
    return (($this->{requirements} & 256)!=0);
  }

  function nongbp () {
    return (($this->{requirements} & 1024)!=0);
  }

  function supportslid () {
    return (($this->{requirements} & 512)!=0);
  }

  function _luhn () {
    $pan=$this->{pan};

    $total = 0;
    $valid = 1;
    $numOfDigits = 0 - strlen($pan);

    $i = -1;
    while ($i >= $numOfDigits){
      if (($i % 2) == 0){
	$double = 2 * (substr($pan, $i, 1));
	$total += substr($double,0,1);
	if (strlen($double > 1)){
	  $total += substr($double,1,1);
	}
      } else {
	$total += substr($pan, $i, 1);
      }
      $i--;
    }

    if (($total % 10) != 0){
      $valid = 0;
    }

    $this->{validnumber}=$valid;
    return ($valid);
  }

  function _cccheck () {

# Check we have located the bin details.  If not, there ain't
# much point in going any further.
    if (!$this->{binfound}) {
      return;
    }

# check length
    $expected_length = $this->digits();
    $mode = $this->{request}->get("Request.Transaction.TxnDetails.mode");
		  
# ficticious test card, let it through
    if ($expected_length == 0 and preg_match("/test/i", $mode)){
      return;
    }
		  
# is it valid - pass luhn check?
    if (!$this->validnumber()) {
      return $this->do_failure("-25", "Bad checksum");
    }

    if (strlen ($this->{pan}) > $expected_length ) {
      return ($this->do_failure("-26", "card number too long"));
    }
    if (strlen ($this->{pan}) < $expected_length) {
      return $this->do_failure("-26", "card number too short");
    }
		  
# check the expiry date
    if (!$this->{request}->get("Request.Transaction.CardTxn.Card.expirydate")) {
      return $this->do_failure("-5", "Expiry date required");
    }
		
    $expiry = $this->{request}->get("Request.Transaction.CardTxn.Card.expirydate");
    if (preg_match("/\d{2}\/\d{2}/", $expiry, $matches) == 0) {
      return $this->do_failure("-23", "Invalid Expiry date");
    }
		  
# check issue number/startdate
    if (!$this->{request}->get("Request.Transaction.CardTxn.Card.issuenumber") and $this->issuenumber() != 0){
      return $this->do_failure("-5", "Issuenumber required");
    } else if ($this->issuenumber() != 0) {
      $issuenumber = $this->{request}->get("Request.Transaction.CardTxn.Card.issuenumber");
      if (preg_match("/^\d+$/", $issuenumber) == 0) {
	return $this->do_failure("-27", "Bad issue");
      }
    }

    if (!$this->{request}->get("Request.Transaction.CardTxn.Card.startdate") and $this->startdate() != 0){
      return $this->do_failure("-5", "Startdate required");
    } else if ($this->{request}->get("Request.Transaction.CardTxn.Card.startdate") != 0) {
      $startdate = $this->{request}->get("Request.Transaction.CardTxn.Card.startdate");
      if (preg_match("/\d{2}\/\d{2}/", $startdate) == 0) {
	return $this->do_failure("-28","Bad start date");
      }
    }
    return;
  }
	
  function validation ($request) {

# Get the DataCash::Request object
    $this->{request} = $request;

  
# Get the PAN before we try to find the binary details, in case
# there are any non-numerics in it
    $this->{pan}= $this->{request}->get("Request.Transaction.CardTxn.Card.pan");
    $this->{pan} = preg_replace("/\D/", "", $this->{pan});
  
# First, let's find the details from the binary file
    $this->_find(0);

    if ($this->{request}->get("Request.Transaction.CardTxn", FALSE)) {
# do credit card checks
      if ($this->{pan} == "") {
	return $this->do_failure("-5", "card number required");
      }
      return $this->_cccheck();
    }
		  
    if ($this->{request}->get("Request.Transaction.ChequeTxn")) {
      $this->_chequecheck();
    }

    if ($this->{request}->get("Request.Transaction.TxnDetails.merchantreference")) {
      $this->_refcheck();
    } else {
      return $this->do_failure("-5", "reference field required");
    }

		  
    return true;
  }
	
  function _refcheck () {
  
    $ref = $this->{request}->get("Transaction.TxnDetails.merchantreference");
    if(strlen ($ref) < 6 || strlen($ref) > 32 || preg_match ("[^A-Za-z0-9]", $ref) == 1) {
      return $this->do_failure("-22","Invalid reference");
    }
		
  }

  function do_failure ($status, $reason) {

    $merchantreference = $this->{request}->get("Request.Transaction.TxnDetails.merchantreference");

    $xml = new DataCash_Document("Response");
    $xml->set("Response.status", $status);
    $xml->set("Response.reason", $reason);
    $xml->set("Response.merchantreference", $merchantreference);

    return $xml;

  }

}

?>
