<?php
	/*
	 * File: content/engine/payment/payment_amazon_checkout.php
	 * Description: The payment method implementation for amazon checkout.
	 * Created: 1440 24/10/2009
	 */
	
	// Details for the payment method loader.
	$payment_processor_id = "amazon_checkout";
	$payment_processor_name = "Amazon Checkout";
	$payment_processor_version = "1.0a";
	$payment_processor_class = "PAYMENT_AMAZON_CHECKOUT";
	$payment_processor_type = "ipn";
	
	// Amazon Cart Class
	class AmazonCart
	{
		/*
		 * Variables
		 */
		private $returnUrl;
		private $cancelUrl;
		private $items = array();
		private $taxTables = array();
		private $shippingAddresses = array();
		private $shipping;
		private $discount = 0;
		
		/*
		 * CTor/DTor
		 */
		
		// CTor
		function __construct()
		{
			global $settings;
			$this->returnUrl = $settings["GlobalHttpsUrl"] . "/" . $settings["INDEX"];
			$this->cancelUrl = $settings["GlobalHttpsUrl"] . "/" . $settings["INDEX"];
		}
		
		/*
		 * Manipulation Functions
		 */
		
		// Sets the return URL. -- Ricky26
		function setReturnURL($url)
		{
			$this->returnUrl = $url;
		}
		
		// Sets the abortion URL. -- Ricky26
		function setCancelUrl($url)
		{
			$this->cancelUrl = $url;
		}
		
		// Sets the shipping price. -- Ricky26
		function setShipping($price)
		{
			$this->shipping = $price;
		}
		
		// Sets the discount amount. -- Ricky26
		function setDiscount($price)
		{
			$this->discount = $price;
		}
		
		// Returns the XML for AWS. -- Ricky26
		function getXML()
		{
			$xml_cart = new SimpleXMLElement('<Order></Order>');
			$xml_cart->addAttribute("xmlns", "http://payments.amazon.com/checkout/2009-05-15/");
			$xml_cart->addChild("ReturnUrl", htmlspecialchars($this->returnUrl));
			$xml_cart->addChild("CancelUrl", htmlspecialchars($this->cancelUrl));
			
			// Assemble the Card
			$cart_node = $xml_cart->addChild("Cart");
			foreach($this->items as $it)
			{
				$this->_addXMLChildren($cart_node->addChild("Item"), $it);
			}
			
			// Assemble the Tax Tables	
			$tax_node = $xml_cart->addChild("TaxTables");
			foreach($this->taxTables as $name => $tax_table)
			{
				$this_node = $tax_node->addChild("TaxTable");
				$this_node->addChild("TaxTableId", htmlspecialchars($name));
				foreach($tax_table as $tax_rule)
				{
					$rule_node = $this_node->addChild("TaxRule");
					$this->_addXMLChildren($rule_node, $tax_rule);
				}				
			}
			
			// Assemble the Shipping Addresses
			$addr_node = $xml_cart->addChild("ShippingAddresses");
			foreach($this->shippingAddresses as $arr)
			{
				$this_node = $addr_node->addChild("ShippingAddress");
				$this->_addXMLChildren($this_node, $arr);
			}
			
			// Assemble the Shipping Methods
			$smeth_node = $xml_cart->addChild("ShippingMethods");
			$this_node = $smeth_node->addChild("ShippingMethod");
			$this->_addXMLChildren($this_node, array(
				"ShippingMethodId"=>"Shipping",
				"ServiceLevel"=>"Standard",
				"Rate"=>array(
					"ShipmentBased"=>$this->shipping,
				),
			));
			
			// Assemble the promotions
			if($this->discount != NULL)
			{
				$smeth_node = $xml_cart->addChild("Promotions");
				$this_node = $smeth_node->addChild("Promotion");
				$this->_addXMLChildren($this_node, array(
					"PromotionId"=>"Discount",
					"Description"=>"Discount",
					"Benefit"=>array(
						"FixedAmountDiscount"=>$this->discount,
					),
				));
				
				$cart_node->addChild("CartPromotionId", htmlentities("Discount"));
			}
			
			return $xml_cart->asXML();
		}
		
		// Converts an array to a set of XML nodes
		function _addXMLChildren($base, $arr)
		{
			foreach($arr as $k => $v)
			{
				if(is_array($v))
				{
					$node = $base->addChild($k);
					$this->_addXMLChildren($node, $v);
				}
				else
				{
					$base->addChild($k, htmlentities($v));
				}
			}
		}
		
		// Add an Item to the cart -- Ricky26
		function addItem($sku, $merchID, $name, $price, $qty=1, $extra=array())
		{
			$this->items[] = array_merge(array(
				"SKU"=>$sku,
				"MerchantId"=>$merchID,
				"Title"=>$name,
				"Price"=>$price,
				"Quantity"=>$qty
			), $extra);
		}
		
		// Make a price array from two parameters. -- Ricky26
		function makePrice($amnt, $currency="USD")
		{
			return array("Amount"=>$amnt, "CurrencyCode"=>$currency);
		}
		
		// Add a shipping address -- Ricky26
		function addShippingAddress($name, $city, $state, $postcode, $countryCode,
		 							$phone, $addressLine1, $addressLine2="", $addressLine3="")
		{
			$ret = array();
			if($addressLine2 != "")
				$ret["AddressFieldTwo"] = $addressLine2;
				
			if($addressLine3 != "")
				$ret["AddressFieldThree"] = $addressLine3;
				
			$ret = array_merge($ret, array(
				"Name"=>$name,
				"City"=>$city,
				"State"=>$state,
				"PostalCode"=>$postcode,
				"CountryCode"=>$countryCode,
				"PhoneNumber"=>$phone,
				"AddressFieldOne"=>$addressLine1,
			));
			
			$this->shippingAddresses[] = $ret;
		}
		
		// Add a tax table
		function addTaxTable($name, $rules)
		{
			$this->taxTables[$name] = $rules;
		}
	}
		
	// Implementation Class
	class PAYMENT_AMAZON_CHECKOUT extends PAYMENT_PROCESSOR
	{
		/*
		 * Variables
		 */
		
		/*
		 * CTor/DTor
		 */
		
		// Constructor
		function PAYMENT_AMAZON_CHECKOUT()
		{
			// Initialise base class
			parent::PAYMENT_PROCESSOR();
				
			// Set up integration details
			$this->id = "amazon_checkout";
			$this->name = "Amazon Simple Pay";
			$this->version = "1.0a";
			$this->class_name = "PAYMENT_AMAZON_CHECKOUT";
			$this->type = "ipn";
			$this->description = "";
			$this->steps = 2;
			$this->testMode = false;
		}
		
		/*
		 *  Methods
		 */
		
		// Turn an error code into a string
		function getErrorString($code)
		{
			$lookup = array(
				"PS"=>"Payment succeeded",
				"PF"=>"Payment failed",
				"RF"=>"Refund failed",
				"RS"=>"Refund succeeded",
				"ME"=>"Server error",
				"A"=>"Payment aborted",
				"SE"=>"Service error",
			);
			if(array_key_exists($code, $lookup))
				return $lookup[$code];
			
			return "Unknown error";
		}
		
		// Get the members of the payment form.
		function getPaymentForm($db)
		{
			global $settings, $order;
			
			// Generate the default payment form. -- Ricky26
			$_pf = parent::getPaymentForm($db);
				
			// Work out URLs here, keeps code clean. -- Ricky26
			$mainUrl = ( $this->protocol=="https" ? $settings["GlobalHttpsUrl"] : $settings["GlobalHttpUrl"] ) . "/";
			$backUrl = $mainUrl.$settings["INDEX"]."?pcsid=".session_id()."&p=cart"; 
			$notifyUrl = $mainUrl.$settings["INDEX"]."?pcsid=".session_id()."&p=".$this->payment_page."&oa=".ORDER_PROCESS_PAYMENT;
			
			// Merchant ID -- Ricky26 
			$mid = $this->settings_vars[$this->id."_merchant_id"]["value"];
			
			// Generate cart for amazon. -- Ricky26
			$cart = new AmazonCart();
			$cart->setReturnUrl($notifyUrl);
			$cart->setCancelUrl($backUrl);
			$cart->setShipping($cart->makePrice($this->common_vars["order_shipping_amount"]["value"]
								+ $this->common_vars["order_handling_amount"]["value"]));
								
			if($this->common_vars["order_discount_amount"]["value"] > 0)
				$cart->setDiscount($cart->makePrice($this->common_vars["order_discount_amount"]["value"]));
								
			if($this->common_vars["shipping_name"]["value"] != "")
			{
				$cart->addShippingAddress(
					$this->common_vars["shipping_name"]["value"],
					$this->common_vars["shipping_city"]["value"],
					$this->common_vars["shipping_state"]["value"],
					$this->common_vars["shipping_zip"]["value"],
					$this->common_vars["shipping_country_iso_a2"]["value"],
					$this->common_vars["billing_phone"]["value"],
					$this->common_vars["shipping_address"]["value"],
					$this->common_vars["shipping_address1"]["value"],
					$this->common_vars["shipping_address2"]["value"]
				);
			}
			else
			{
				$cart->addShippingAddress(
					$this->common_vars["shipping_company"]["value"],
					$this->common_vars["shipping_city"]["value"],
					$this->common_vars["shipping_state"]["value"],
					$this->common_vars["shipping_zip"]["value"],
					$this->common_vars["shipping_country_iso_a2"]["value"],
					$this->common_vars["billing_phone"]["value"],
					$this->common_vars["shipping_address"]["value"],
					$this->common_vars["shipping_address1"]["value"],
					$this->common_vars["shipping_address2"]["value"]
				);				
			}
			
			// Variables
			$tax_rates = array();
			
			// Get items from cart -- Ricky26
			$db->query("
				SELECT 
					".DB_PREFIX."orders_content.*, 
					".DB_PREFIX."products.description AS description,
					".DB_PREFIX."products.is_taxable AS product_taxable,
					".DB_PREFIX."products.tax_class_id AS tax_class_id,
					".DB_PREFIX."products.tax_rate AS product_tax_rate,
					".DB_PREFIX."tax_classes.class_name AS tax_class_name
				FROM ".DB_PREFIX."orders_content 
				INNER JOIN ".DB_PREFIX."products ON ".DB_PREFIX."products.pid = ".DB_PREFIX."orders_content.pid
				LEFT JOIN ".DB_PREFIX."tax_classes ON (".DB_PREFIX."tax_classes.class_id = ".DB_PREFIX."products.tax_class_id)	
				WHERE oid='".$this->common_vars["order_oid"]["value"]."';");
			while($db->moveNext())
			{
				if($db->col["product_taxable"])
				{
					if($db->col["product_tax_rate"] > 0)
					{
						$cart->addTaxTable("pid_" . $db->col["product_id"], array(
							array(
								"Rate"=>$db->col["product_tax_rate"],
								"PreDefinedRegion"=>"WorldAll",
							)
						));
						$cart->addItem(
							$db->col["product_id"],
							$mid, $db->col["title"],
							$cart->makePrice($db->col["price"]),
							$db->col["quantity"],
							array("TaxTableId"=>"pid_" . $db->col["product_id"]));
					}
					else
					{
						$tax_rate = array("class_name"=>"none");
						$tcid = $db->col["tax_class_id"];
						if(!array_key_exists($tcid, $tax_rates))
						{
							$db->query("SELECT * FROM ".DB_PREFIX."tax_classes WHERE class_id = '".$tcid."';");
							foreach($db->getRecords() as $row)
							{
								$tax_rates[$tcid] = $row;
								$tax_rate = $row;
							}
						}
						else
							$tax_rate = $tax_rates[$tcid];
							
						// FOR DEBUGGING -- Ricky26
						/*$toPrint = "";
						$this->logStart();
						foreach($tax_rate as $k=>$v)
							$toPrint = $toPrint . $k . " = " . $v . "\n";
						$this->log("TAX_RATE: " . $toPrint);
						$this->logFinish();*/
						
						$cart->addItem(
							$db->col["product_id"],
							$mid, $db->col["title"],
							$cart->makePrice($db->col["price"]),
							$db->col["quantity"],
							array("TaxTableId"=>$tax_rate["class_name"]));
					}
				}
				else
					$cart->addItem(
						$db->col["product_id"],
						$mid, $db->col["title"],
						$cart->makePrice($db->col["price"]),
						$db->col["quantity"]);
			}
			
			foreach($tax_rates as $taxrate)
			{
				// Sort out actual tax rates
				$db->query("
					SELECT 
						".DB_PREFIX."tax_rates.tax_rate,
						".DB_PREFIX."tax_zones_regions.coid as country_id,						
						".DB_PREFIX."tax_zones_regions.stid as state_id,
						if(".DB_PREFIX."tax_zones_regions.coid = 0, 2, 1) AS country_priority,
						if(".DB_PREFIX."tax_zones_regions.stid = 0, 2, 1) AS state_priority,
						".DB_PREFIX."countries.iso_a2 AS country_code,
						".DB_PREFIX."states.short_name AS state_code
					FROM ".DB_PREFIX."tax_rates 
					INNER JOIN ".DB_PREFIX."tax_zones_regions ON (".DB_PREFIX."tax_zones_regions.zone_id = ".DB_PREFIX."tax_rates.zone_id)
					LEFT JOIN ".DB_PREFIX."countries ON (".DB_PREFIX."countries.coid =".DB_PREFIX."tax_zones_regions.coid)
				   	LEFT JOIN ".DB_PREFIX."states ON (".DB_PREFIX."states.stid = ".DB_PREFIX."tax_zones_regions.stid)
					WHERE ".DB_PREFIX."tax_rates.class_id='".$tax_rate["class_id"]."' 
					ORDER BY ".DB_PREFIX."tax_rates.rate_priority ASC, country_priority ASC, state_priority ASC
				");
				if($db->numRows() > 0)
				{
					$rates = array();
					while($db->moveNext())
					{
						// Calculate rate as a number from 0 to 1
						$rate = array("Rate"=>$db->col["tax_rate"]/100.0);
						
						if($db->col["country_id"] == 0)
						{
							// World Wide
							$rate["PreDefinedRegion"] = "WorldAll";
						}
						else if($db->col["country_code"] == "US")
						{
							if($db->col["state_id"] == 0)
							{
								// Whole US
								$rate["PreDefinedRegion"] = "USAll";
							}
							else
							{
								// A state in the US
								$rate["USStateRegion"] = $db->col["state_code"];
							}
						}
						else
						{
							// Another country
							$rate["WorldRegion"] = array("CountryCode"=>$db->col["country_code"]);
						}
						
						$rates[] = $rate;
					}
					
					$cart->addTaxTable($taxrate["class_name"], $rates);
				}
			}
			
			// Generate XML for cart. -- Ricky26
			$xml_cart = $cart->getXML();
			
			// Generate fields. -- Ricky26
			$fields = array();			
			
			// FOR DEBUGGING -- Ricky26
			$this->logStart();
			$this->log("Amazon Cart[".$this->common_vars["order_oid"]["value"]."]:\n" . $xml_cart);
			$this->logFinish();
				
			$fields[] = array("type"=>"hidden", "name"=>"order-input", "value"=>"type:merchant-signed-order/aws-accesskey/1;order:"
				. base64_encode($xml_cart)
				. ";signature:" . $this->hashHMAC($xml_cart, $this->settings_vars[$this->id."_secret_key"]["value"])
				. ";aws-access-key-id:" . $this->settings_vars[$this->id."_access_key"]["value"]
				);
			
			$fields[] = array("type"=>"hidden", "name"=>"signature", "value"=>$this->hashHMAC($xml_cart, $this->settings_vars[$this->id."_secret_key"]["value"]));

			return $fields;
		}
		
		function isTestMode()
		{
			$this->testMode = $this->settings_vars[$this->id."_test_mode"]['value'] != "No";
			return $this->testMode;
		}
		
		// Get JS code for form validation
		function getValidatorJS()
		{
			// We don't need any validation as we're forwarding control to
			// Amazon... -- Ricky26
			return "";
		}
		
		// Get the url to forward to.
		function getPostUrl()
		{
			// We want to return either their custom URL, the sandbox URL or the
			// standard URL for amazon checkout. -- Ricky26
			
			$postUrl = "";
			$mid = $this->settings_vars[$this->id."_merchant_id"]["value"];
			
			if($this->url_to_gateway != "")
				$postUrl = trim($this->url_to_gateway);	
			elseif($this->isTestMode())
				$postUrl = "https://payments-sandbox.amazon.com/checkout/$mid?order-channel=apm";
			else
				$postUrl = "https://payments.amazon.com/checkout/$mid?order-channel=apm";
			
			$this->post_url = $postUrl;
			return $postUrl;
		}
		
		// Process the result from the AWS.
		function process($db, $user, $order, $post_form)
		{			
			// Variables for the signature -- Ricky26
			$sig = NULL;
			$sigText = "";
			$debug = FALSE;
			
			// Merge and sort the GET/POST variables -- Ricky26
			$vars = array_merge(array(), $_GET, $_POST);
			$varNames = array_keys($vars);
			natcasesort($varNames);
			
			// Generate the text for encoding -- Ricky26
			$logText = "\nAmazon vars:\n";
			foreach($varNames as $k)
			{
				$v = $vars[$k];
				
				$logText = $logText . ("\t" . $k . ":\t" . $v . "\n");
				
				// Check for signature, and signature details
				if($k == "signature")
					$sig = $v;
				else
					$sigText = $sigText . $k . $v;
			}
			
			// Check signature -- Ricky26
			$sigCheck = $this->hashHMAC($sigText, $this->settings_vars[$this->id."_secret_key"]["value"]);
			$OK = strcasecmp($sigCheck, $sig) == 0;
			
			// Logging
			if(!$OK || $debug)
			{	
				$this->logStart();
				$this->log($logText);
				
				if(!$OK)
					$this->log("Failed attempt.\n" . file_get_contents("php://input") . "\n" . $sigText . "\n" . $sigCheck);
				$this->logFinish();
			}
			
			// If the signature failed
			// we can't have received payment. -- Ricky26
			if(!$OK)
			{
				$this->is_error = TRUE;
				$this->error_message = "Payment signature failure, Amazon Payments gateway failure.";
				return !$this->is_error;
			}
			
			$status = $vars["status"];
			
			// Support for delayed transactions. -- Ricky26
			if($status == "PI")
			{
				$this->is_error = FALSE;
				$this->error_message = "Payment is pending, your order will be shipped once payment is received.";
				return "Pending";
			}
			
			// Return whether it was a success -- Ricky26
			if($status == "PS")
			{				
				$this->is_error = FALSE;
				$this->error_message = "";
				$this->createTransaction($db, $user, $order, $sigText, $vars["transactionId"]);
			}
			else
			{
				$this->logStart();
				$this->log("Failed payment:\n" . $logText);
				$this->logFinish();
				
				$this->is_error = TRUE;
				$this->error_message = $this->getErrorString($status) . ", please try again.";
			}
			
			return !$this->is_error;
		}
		
		// Generate a HMAC hash.
		function hashHMAC($data, $key)
		{
			$sig = hash_hmac("sha1", $data, $key, TRUE);
			$sig = base64_encode($sig);
			return $sig;
		}
	}
?>