<?php 
	$payment_processor_id = "hsbccpi";
	$payment_processor_name = "HSBC-CPI";
	$payment_processor_version = "3.3";
	$payment_processor_class = "PAYMENT_HSBCCPI";
	$payment_processor_type = "ipn";
	
	class PAYMENT_HSBCCPI extends PAYMENT_PROCESSOR{
		//currencies
		var $currency = array(
			"Euro"=>"978",
			"Hong Kong Dollar"=>"344",
			"Japanese Yen"=>"392",
			"Pound Sterling"=>"826",
			"US Dollar"=>"840"
		);
		
		//////////////////////////////////////
		// Constructor
		function PAYMENT_HSBCCPI(){
			parent::PAYMENT_PROCESSOR();
			$this->id = "hsbccpi";
			$this->name = "HSBC-CPI";
			$this->version = "3.3";
			$this->class_name = "PAYMENT_HSBCCPI";
			$this->type = "ipn";
			$this->description = "";
			$this->post_url = "https://www.cpi.hsbc.com/servlet";
			$this->steps = 2;
			
			return $this;
		}
		
		//////////////////////////////////////
		// Return a Hash Key
		function getHash($data, $key, $system){
			global $settings;
			//determinate operation system
			$path = $settings["GlobalServerPath"]."/content/engine/payment/hsbccpi/";
			if(($system == "AutoDetect" && preg_match("/Win32/i", $_SERVER["SERVER_SOFTWARE"]))||$system == "Windows"){
				$exe_path = $path."win32/TestHash.exe";
				$exe_path = str_replace('\\\\', '\\', $exe_path);
				putenv("LD_LIBRARY_PATH=".str_replace('\\\\', '\\', $path)."win32");
			}
			if(($system == "AutoDetect" && preg_match("/Linux/i", $_SERVER["SERVER_SOFTWARE"]))||$system == "Linux"){
				$exe_path = $path."linux/TestHash.e";
				putenv("LD_LIBRARY_PATH=".$path."linux");
			}
			if(($system == "AutoDetect" && preg_match("/Unix/i", $_SERVER["SERVER_SOFTWARE"]))||$system == "SunOS(Unix)"){
				$exe_path = $path."sunos/TestHash.e";
				putenv("LD_LIBRARY_PATH=".$path."sunos");
			}
			if(($system == "AutoDetect" && preg_match("/HP-UX/i", $_SERVER["SERVER_SOFTWARE"]))||$system == "HP-UX"){
				$exe_path = $path."hp-ux/TestHash.e";
				putenv("LD_LIBRARY_PATH=".$path."hp-ux");
			}
			//get param str and execute command
			$param_str = "";
			foreach($data as $_key=>$_value){
				$_value = trim($_value);
				$param_str.=' "'.$_value.'"';
			}
			$cmd = $exe_path." ".$key.' '.trim($param_str);
			$return = exec($cmd);
			$parts = split(":", $return);
			//return hash
			return trim($parts[1]);
		}
		
		/////////////////////////////
		// Redeclare getPaymentForm
		function getPaymentForm($db){
			global $settings, $order;
			global $msg;
			$_pf = parent::getPaymentForm($db);
			
			$notify_url = ($this->protocol=="https"?$settings["GlobalHttpsUrl"]:$settings["GlobalHttpUrl"])."/".$settings["INDEX"]."?pcsid=".session_id()."&p=".$this->payment_page."";
			$return_url = ($this->protocol=="https"?$settings["GlobalHttpsUrl"]:$settings["GlobalHttpUrl"])."/".$settings["INDEX"]."?pcsid=".session_id()."&p=".$this->payment_page."&oa=".ORDER_PROCESS_PAYMENT;
			
			//$notify_url = ($this->protocol=="https"?$settings["GlobalHttpsUrl"]:$settings["GlobalHttpUrl"])."/".$settings["INDEX"]."?p=invoice";
			//$return_url = ($this->protocol=="https"?$settings["GlobalHttpsUrl"]:$settings["GlobalHttpUrl"])."/".$settings["INDEX"]."?p=invoice&oa=ProcessPayment";
			
			$data = array();
			
			$data["CpiDirectResultUrl"] = $notify_url;
			$data["CpiReturnUrl"] = $return_url;
			
			$data["MerchantData"] = session_id();
			if($this->settings_vars[$this->id."_Mode"]["value"] == "Test"){
				$data["Mode"] = $this->settings_vars[$this->id."_Mode"]["value"] == "Test"?"T":"P";
			}
			$data["OrderDesc"] = "Order ".$order->order_num;
			$data["OrderId"] = $order->order_num;
			$data["PurchaseAmount"] = round($this->common_vars["order_total_amount"]["value"] * 100);
			$data["PurchaseCurrency"] = $this->currency[$this->settings_vars[$this->id."_Currency"]["value"]];
			$data["StorefrontId"] = $this->settings_vars[$this->id."_Storefront_Id"]["value"];
			//$data["TimeStamp"] = strtotime("-0 hours 1 minutes")."00";
			$data["TimeStamp"] = gmdate("U") * 1000;//time() + date('Z', time()))*1000;
			$data["TransactionType"] = $this->settings_vars[$this->id."_Transaction_Type"]["value"];
			//$data["UserId"] = $this->common_vars["customer_id"]["value"];
			
			$data["BillingAddress1"] = $this->common_vars["billing_address1"]["value"];
			$data["BillingAddress2"] = $this->common_vars["billing_address2"]["value"];
			$data["BillingCity"] = $this->common_vars["billing_city"]["value"];
			$data["BillingCounty"] = $this->common_vars["billing_state"]["value"];
			$data["BillingCountry"] = $this->common_vars["billing_country_iso_number"]["value"];
			$data["BillingFirstName"] = $this->common_vars["billing_first_name"]["value"];
			$data["BillingLastName"] = $this->common_vars["billing_last_name"]["value"];
			$data["BillingPostal"] = $this->common_vars["billing_zip"]["value"];
			$data["ShopperEmail"] = $this->common_vars["billing_email"]["value"];
			
			if(!$this->common_vars["shipping_digital"]["value"]){
				$data["ShippingAddress1"] = $this->common_vars["shipping_address1"]["value"];
				$data["ShippingAddress2"] = $this->common_vars["shipping_address2"]["value"];
				$data["ShippingCity"] = $this->common_vars["shipping_city"]["value"];
				$data["ShippingCounty"] = $this->common_vars["shipping_state"]["value"];
				$data["ShippingCountry"] = $this->common_vars["shipping_country_iso_number"]["value"];
				$data["ShippingFirstName"] = $this->common_vars["shipping_name"]["value"];
				$data["ShippingLastName"] = $this->common_vars["shipping_name"]["value"];
				$data["ShippingPostal"] = $this->common_vars["shipping_zip"]["value"];
			}
			
			$order_hash = $this->getHash(
				$data, 
				$this->settings_vars[$this->id."_Hash_Key"]["value"], 
				$this->settings_vars[$this->id."_Server_Os"]["value"]
			);
			
			$fields = array();
			$pf = "";
			foreach($data as $key=>$value){
				$fields[] = array("type"=>"hidden", "name"=>$key, "value"=>$value);
			}
		
			$fields[] = array("type"=>"hidden", "name"=>"OrderHash", "value"=>$order_hash);
		
			return $fields;
		}

		function getValidatorJS(){
			return "";
		}
		
		function isTestMode(){
			if($this->settings_vars[$this->id."_Mode"]["value"] == "TEST"){
				$this->testMode = true;
			}
			else{
				$this->testMode = false;
			}
			return $this->testMode;
		}
		
		function getPostUrl(){
			if(trim($this->url_to_gateway) != ""){
				return trim($this->url_to_gateway);
			}
			else{
				return $this->post_url;
			}
		}
		
		
		//////////////////////////////////////////
		// Process Transaction
		function process($db, $user, $order, $post_form){
			global $settings;
			
			if($post_form["CpiResultsCode"] == "0"){
				//check order hash
				$data = array();
				$response = "";
				foreach($post_form as $var_name=>$var_value){
					if($var_name != "OrderHash"){
						$data[$var_name] = $var_value;
					}
					$response.=$var_name." = ".$var_value."\n";
				}
				
				$order_hash = $this->getHash(
					$data, 
					$this->settings_vars[$this->id."_Hash_Key"]["value"], 
					$this->settings_vars[$this->id."_Server_Os"]["value"]
				);
				
				$this->createTransaction($db, $user, $order, $response, "");
				$this->is_error = false;
				$this->error_message = "";
			}
			else{
				$error_messages = array(
					"1"=>"The user cancelled the transaction.",
					"2"=>"The processor declined the transaction for an unknown reason.",
					"3"=>"The transaction was declined because of a problem with the card. For example, an invalid card number or expiration date was specified.",
					"4"=>"The processor did not return a response.",
					"5"=>"The amount specified in the transaction was either too high or too low for the processor.",
					"6"=>"The specified currency is not supported by either the processor or the card.",
					"7"=>"The order is invalid because the order ID is a duplicate.",
					"8"=>"The transaction was rejected by FraudShield.",
					"9"=>"The transaction was placed in Review state by FraudShield.",
					"10"=>"The transaction failed because of invalid input data.",
					"11"=>"The transaction failed because the CPI was configured incorrectly.",
					"12"=>"The transaction failed because the Storefront was configured incorrectly.",
					"13"=>"The connection timed out.",
					"14"=>"The transaction failed because the cardholder's browser refused a cookie.",
					"15"=>"The customer's browser does not support 128-bit encryption.",
					"16"=>"The CPI cannot communicate with the Secure ePayment engine."
				);
				$this->is_error = true;
				$this->error_message = $error_messages[$post_form["CpiResultsCode"]];
			}
			return !$this->is_error;
		}
	}
?>
