<?php
	$payment_processor_id = "paypalwppuk";
	$payment_processor_name = "PayPal Website Payments Pro (Payflow Edition)";
	$payment_processor_version = "0.1";
	$payment_processor_class = "PAYMENT_PAYPALWPPUK";
	$payment_processor_type = "cc";
	
	class PAYMENT_PAYPALWPPUK extends PAYMENT_PROCESSOR
	{
		function PAYMENT_PAYPALWPPUK()
		{
			parent::PAYMENT_PROCESSOR();
			$this->id = "paypalwppuk";
			$this->name = "PayPal Website Payments Pro (Payflow Edition)";
			$this->version = "3.0";
			$this->class_name = "PAYMENT_PAYPALWPPUK";
			$this->type = "cc";
			$this->description = "";
			$this->steps = 1;
			$this->testMode = false;
			$this->requires_cc_start_date = true;
			$this->requires_issue_number  = true;
			$this->support_ccs = true;
			$this->need_cc_codes = true;
			return $this;
		}
		
		/* Get Payment Form
		*/
		function getPaymentForm($db)
		{
			global $settings;
			$fields = parent::getPaymentForm($db);
			if ($settings["cardinalcommerce_Enabled"] == "Yes" && $this->settings_vars[$this->id."_Enable_Cardinal_Centinel"]["value"] == "Yes")
			{
				$fields["do_cmpi_lookup"] = array("type" => "hidden", "name"=> "do_cmpi_lookup", "value" => 1);
			}
			return $fields;
		}
		
		/* Check is module in test mode
		*/
		function isTestMode()
		{
			if ($this->settings_vars[$this->id."_Test_Mode"]["value"] == "YES")
			{
				$this->testMode = true;
			}
			else{
				$this->testMode = false;
			}
			return $this->testMode;
		}
		
		/* Send data to servers via cURL / HTTPS
		*/
		function process_curl($post_data, $post_url)
		{
			global $settings;
			$c = curl_init($post_url);
				
			if ($settings["ProxyAvailable"] == "YES")
			{
				//curl_setopt($c, CURLOPT_VERBOSE, 1);
				if (defined("CURLOPT_PROXYTYPE") && defined("CURLPROXY_HTTP") && defined("CURLPROXY_SOCKS5"))
				{
					curl_setopt($c, CURLOPT_PROXYTYPE, $settings["ProxyType"] == "HTTP" ? CURLPROXY_HTTP : CURLPROXY_SOCKS5);
				}
				curl_setopt($c, CURLOPT_PROXY, $settings["ProxyAddress"].":".$settings["ProxyPort"]);
				if ($settings["ProxyRequiresAuthorization"] == "YES")
				{
					curl_setopt($c, CURLOPT_PROXYUSERPWD, $settings["ProxyUsername"].":".$settings["ProxyPassword"]);
				}

				curl_setopt($c, CURLOPT_TIMEOUT, 120);
			}
				
			
			$headers = array(
				"Content-Length: ".strlen($post_data),
				"Content-Type: text/namevalue",
				"Host: pilot-payflowpro.verisign.com",
				"X-VPS-REQUEST-ID: ".md5(uniqid(rand(),1)),
				"X-VPS-CLIENTTIMEOUT: 45",
				"X-VPS-VITCLIENTCERTIFICATION-ID: ",
				"X-VPS-VITINTEGRATION-PRODUCT: Shopping Cart",
				"X-VPS-VITINTEGRATION-VERSION: ".$this->version
			);
			
			curl_setopt($c, CURLOPT_HTTPHEADER, $headers);

			curl_setopt($c, CURLOPT_VERBOSE, 0);
			curl_setopt($c, CURLOPT_HEADER, 0);
			curl_setopt($c, CURLOPT_POSTFIELDS, $post_data);
			curl_setopt($c, CURLOPT_POST, 1);
			curl_setopt($c, CURLOPT_RETURNTRANSFER, 1);
			if(!ini_get('safe_mode')){
				set_time_limit(3000);
			}
			curl_setopt($c, CURLOPT_SSL_VERIFYPEER, 1);
			curl_setopt($c, CURLOPT_CAPATH, $settings['GlobalServerPath'].$settings['SecuritySslDirectory']);
			curl_setopt($c, CURLOPT_CAINFO, $settings['GlobalServerPath'].$settings['SecuritySslDirectory'].'/'.$settings['SecuritySslPem']);
			$buffer = curl_exec($c);
			
			if (curl_errno($c) > 0)
			{
				$this->is_error = true;
				$this->error_message = curl_error($c);
				return false;
			}
			
			if (!$buffer)
			{
				return false;
			}
			return $buffer;
		}
		
		function processSale($db, $user, $order, $post_form, $cardinalData = false)
		{
			global $settings;
			//build post data for query
			$post_data = "";
			// get post_form, common, settings and custom vars
			reset($this->common_vars);
			$post_data = "".
				"USER=".urlencode($this->settings_vars[$this->id."_User"]["value"])."&".
				"VENDOR=".urlencode($this->settings_vars[$this->id."_Vendor"]["value"])."&".
				"PARTNER=".urlencode($this->settings_vars[$this->id."_Partner"]["value"])."&".
				"PWD=".urlencode($this->settings_vars[$this->id."_Password"]["value"])."&".
				"TENDER=".urlencode("C")."&". //C = Credit card for Direct Payment transactions
				"TRXTYPE=".urlencode("S")."&". //S = Sale transaction
				
				//CC DATA
				"ACCT=".urlencode($post_form["cc_number"])."&".				
				(array_key_exists("cc_type", $post_form) && in_array($post_form["cc_type"], array("0", "1", "8", "9", "S"))?("ACCTTYPE=".urlencode($post_form["cc_type"])."&"):"").
				"AMT=".urlencode(number_format($this->common_vars["order_total_amount"]["value"], 2, ".", ""))."&".
				
				//BILLING INFO
				"STREET=".urlencode($this->common_vars["billing_address"]["value"])."&".
				"CITY=".urlencode($this->common_vars["billing_city"]["value"])."&".
				"STATE=".urlencode($this->common_vars["billing_state"]["value"])."&".
				
				"COUNTRY=".urlencode($this->common_vars["billing_country_iso_a2"]["value"])."&".
				"ZIP=".urlencode($this->common_vars["billing_zip"]["value"])."&".
				
				"BUTTONSOURCE=".urlencode($this->common_vars["order_id"]["value"])."&";
			if (array_key_exists("cc_type", $post_form) && in_array($post_form["cc_type"], array("9", "S")))
			{	
				$post_data.=
					"CARDSTART=".urlencode($post_form["cc_start_month"].substr($post_form["cc_start_year"], 2, 2))."&".
					"CARDISSUE=".urlencode($post_form["cc_issue_number"])."&";
			}
				
				
			$post_data.=	
				"CLIENTIP=".urlencode($_SERVER["REMOTE_ADDR"])."&".
				"COMMENT1=".urlencode("")."&".
				"COMMENT2=".urlencode("")."&".
				"CAPTURECOMPLETE=Y&".
				
				//USD, EUR, GBP, CAD, JPY, AUD
				"CURRENCY=".urlencode($this->settings_vars[$this->id."_Currency"]["value"])."&".
				
				"CUSTOM=".urlencode("")."&".
				"CUSTREF=".urlencode($this->common_vars["order_id"]["value"])."&".
				
				//The value of CVV2 is required for Visa, MasterCard, Switch, and Solo cards.
				"CVV2=".urlencode($post_form["cc_cvv2"])."&".
				"EMAIL=".urlencode($this->common_vars["billing_email"]["value"])."&".
				"PHONENUM=".urlencode($this->common_vars["billing_phone"]["value"])."&".
				"EXPDATE=".urlencode($post_form["cc_expiration_month"].substr($post_form["cc_expiration_year"], 2, 2))."&".
				
				"FREIGHTAMT=".urlencode("0.00")."&".
				"HANDLINGAMT=".urlencode("0.00")."&".
				
				"INVNUM=".urlencode($this->common_vars["order_id"]["value"])."&".
				
				//ITEMAMT - TODO
				//L_DESCn
				//L_AMTn
				//L_QTYn
				//L_TAXAMTn
				//MERCHANTSESSIONID
				
				"FIRSTNAME=".urlencode($post_form["cc_first_name"])."&".
				"LASTNAME=".urlencode($post_form["cc_last_name"])."&";
				
				//NOTIFYURL - TODO
				//ORDERDESC - TODO
				//ORIGID - TODO
				//RECURRINGTYPE - TODO
				
			//shipping address
			if (!$order->shippingDigital)
			{
				$post_data.= 
					"SHIPTOSTREET=".urlencode($this->common_vars["shipping_address"]["value"])."&".
					"SHIPTOCITY=".urlencode($this->common_vars["shipping_city"]["value"])."&".
					"SHIPTOSTATE=".urlencode($this->common_vars["shipping_state"]["value"])."&".
					"SHIPTOCOUNTRY=".urlencode($this->common_vars["shipping_country_iso_a2"]["value"])."&".
					"SHIPTOZIP=".urlencode($this->common_vars["shipping_zip"]["value"])."&";
			}
			$post_data.=
				"FREIGHTAMT=".urlencode("0.00")."&".
				"VERBOSITY=MEDIUM";			
			
			if ($cardinalData)
			{
				$post_data.=$cardinalData;
			}
			
			$post_url = trim($this->url_to_gateway);
			if ($post_url == "")
			{
				$post_url = $this->isTestMode() ? "https://pilot-payflowpro.verisign.com" : "https://payflowpro.verisign.com/transaction";
			}
			
			$response = $this->process_curl($post_data, $post_url);
			if ($response)
			{
				$_parts = explode("&", $response);
				$_result = array();
				for ($i=0; $i<count($_parts); $i++)
				{
					$_pair = explode("=", $_parts[$i]);
					if (count($_pair) == 1)
					{
						$_result[$_pair[0]] = "";
					}
					if (count($_pair) == 2)
					{
						$_result[$_pair[0]] = $_pair[1];
					}
				}
				$result = isset($_result["RESULT"])?$_result["RESULT"]:"";
				switch ($result)
				{
					case "0" : 
					{ // approved
						$this->createTransaction($db, $user, $order, str_replace("&", "\n", $response), "");
						if ($this->support_ccs && $this->enable_ccs)
						{
							$card_data = array(
								"fn"=>$post_form["cc_first_name"],
								"ln"=>$post_form["cc_last_name"],
								"ct"=>$post_form["cc_type"],
								"cc"=>$post_form["cc_number"],				
								"em"=>$post_form["cc_expiration_month"], 
								"ey"=>$post_form["cc_expiration_year"]
							);
							$this->saveCCdata($db, $order, $card_data, base64_decode($settings["SecurityCCSCertificate"]));
						}
						$this->is_error = false;
						$this->error_message = "";
						return "Received";
						break;
					}
					default :
					{
						$respmsg = isset($_result["RESPMSG"])?$_result["RESPMSG"]:PAYMENT_TRANSACTION_ERROR_TEXT;
						$this->is_error = true;
						$this->error_message = $respmsg;
						break;
					}
				}
			}
			
			return !$this->is_error;
		}
		
		//main payment module entry point
		function process($db, $user, $order, $post_form)
		{	
			
			global $settings;
			global $backurl;
			
			//check cardinal centinel is enabled
			if ($settings["cardinalcommerce_Enabled"] == "Yes" && $this->settings_vars[$this->id."_Enable_Cardinal_Centinel"]["value"] == "Yes")
			{
				//include module
				require_once("content/engine/engine_cardinal_commerce.php");
				//create object and set it up
				$cardinal = new CardinalCommerce();
				$cardinal->version ="1.7";
				$cardinal->common_vars = $this->common_vars;
				if (isset($_REQUEST["do_cmpi_lookup"]))
				{
					$cardinal_lookup_xml = $cardinal->getLookupXML(
						$post_form["cc_first_name"],
						$post_form["cc_last_name"],	
						$post_form["cc_number"], 
						$post_form["cc_expiration_month"], 
						$post_form["cc_expiration_year"],
						$this->currency["code"]
					);
					
					$cardinal_lookup_result = $cardinal->processRequest($cardinal_lookup_xml);
					if ($cardinal_lookup_result)
					{
						if ($cardinal_lookup_result["CardinalMPI"][0]["Enrolled"][0] == "Y")
						{
							//change back url
							$backurl = ($this->protocol=="https"?$settings["GlobalHttpsUrl"]:$settings["GlobalHttpUrl"])."/".$settings["INDEX"]."?pcsid=".session_id()."&p=payment_validation";
							
							$_SESSION["CardinalPostForm"] = $post_form;
							$_SESSION["CardinalTransactionId"] = $cardinal_lookup_result["CardinalMPI"][0]["TransactionId"][0];							
							$_SESSION["CardinalACSUrl"] = $cardinal_lookup_result["CardinalMPI"][0]["ACSUrl"][0];
							$_SESSION["CardinalPayload"] = $cardinal_lookup_result["CardinalMPI"][0]["Payload"][0];
							//validator will replace ORDER_PROCESS_PAYMENT_VALIDATION with ORDER_PROCESS_PAYMENT
							$_SESSION["CardinalNotifyUrl"] =($this->protocol=="https"?$settings["GlobalHttpsUrl"]:$settings["GlobalHttpUrl"])."/".$settings["INDEX"]."?pcsid=".session_id()."&p=".$this->payment_page."&oa=".ORDER_PROCESS_PAYMENT_VALIDATION."&do_cmpi_authenticate=true";
							$_SESSION["CardinalRedirectUrl"] = ($this->protocol=="https"?$settings["GlobalHttpsUrl"]:$settings["GlobalHttpUrl"])."/".$settings["INDEX"];
							return;
						}
						else
						{
							//do not redirect, process simple authorize transaction
							$this->processSale($db, $user, $order, $post_form);
						}
					}
					else
					{
						$this->is_error = true; 
						$this->error_message = $cardinal->error_message;
					}
				}
				if (isset($_REQUEST["do_cmpi_authenticate"]))
				{
					$post_form = $_SESSION["CardinalPostForm"];
					unset($_SESSION["CardinalPostForm"]);
					$cardinal_auth_xml = $cardinal->getAuthenticateXML($_REQUEST["PaRes"]);
					$cardinal_auth_result = $cardinal->processRequest($cardinal_auth_xml);
					if ($cardinal_auth_result)
					{
						$EciFlag = $cardinal_auth_result["CardinalMPI"][0]["EciFlag"][0];
						$PAResStatus = $cardinal_auth_result["CardinalMPI"][0]["PAResStatus"][0];
						$Cavv = $cardinal_auth_result["CardinalMPI"][0]["Cavv"][0];
						$SignatureVerification = $cardinal_auth_result["CardinalMPI"][0]["SignatureVerification"][0];
						$Xid = $cardinal_auth_result["CardinalMPI"][0]["Xid"][0];
						$ErrorNo = $cardinal_auth_result["CardinalMPI"][0]["ErrorNo"][0];
						if ($SignatureVerification == "Y" && $PAResStatus != "N")
						{
							switch ($PAResStatus)
							{
								case "Y" : 
								case "A" :
								{
									$cardinalData = 
										"&AUTHSTATUS3D=".$PAResStatus.
										"&MPIVENDOR3DS=Y".
										"&CAVV=".$Cavv.
										"&ECI=".$EciFlag.
										"&XID=".$Xid;
									$this->processSale($db, $user, $order, $post_form, $cardinalData);
									break;
								}
								case "U" : $this->processSale($db, $user, $order, $post_form); break;
							}
						}
						else 
						{
							$this->is_error = true;
							$this->error_message = "Your card can not be validated. Please try other card or other payment method";
						}
					}
					else
					{
						$this->is_error = true;
						$this->error_message = $cardinal->error_message;
					}
				}
			}
			else
			{
				$this->processSale($db, $user, $order, $post_form);
			}
			
			return !$this->is_error;
		}
	}
