<?php 
	$payment_processor_id = "qbms";
	$payment_processor_name = "QuickBooks Merchant Services";
	$payment_processor_version = "3.3";
	$payment_processor_class = "PAYMENT_QBMS";
	$payment_processor_type = "cc";
	
	class PAYMENT_QBMS extends PAYMENT_PROCESSOR
	{
		
		function PAYMENT_QBMS(){
			parent::PAYMENT_PROCESSOR();
			$this->id = "qbms";
			$this->name = "QuickBooks Merchant Services";
			$this->version = "3.3";
			$this->class_name = "PAYMENT_QBMS";
			$this->type = "cc";
			$this->description = "";
			$this->steps = 1;
			$this->testMode = false;
			$this->need_cc_codes = false;
			$this->support_ccs = false;
			return $this;
		}
		
		function getPaymentForm($db){
			$fields = parent::getPaymentForm($db);
			
			return $fields;
		}
		
		function isTestMode(){
			static $testMode = null;
			
			if (!is_null($testMode)) return $testMode;
			$testMode = ($this->settings_vars[$this->id."_test_mode"]["value"] == "On");
			$this->testMode = $testMode;
			
			return $this->testMode;
		}
		
		// process transaction
		
		/**
		 * Processes Other Transactions
		 *
		 * @param DB $db 
		 * @param string $custom_id 
		 * @param string $order_id 
		 * @return void
		 * @author Sebastian Nievas
		 */
		function processCustom($db, $custom_id, $order_id)
		{
			global $settings;
			
			$order_id = intval(is_numeric($order_id) ? $order_id : 0);

			if (!$order_id)
			{
				// throw new Exception("An Error Has Occurred - Invalid Order ID");
			}
			
			$db->query("
				SELECT ".DB_PREFIX."orders.*,
					".DB_PREFIX."countries.name AS country_name,
					".DB_PREFIX."countries.iso_a2 AS country_iso_a2,
					".DB_PREFIX."countries.iso_a3 AS country_iso_a3,
					".DB_PREFIX."countries.iso_number AS country_iso_number
				FROM ".DB_PREFIX."orders 
				LEFT JOIN ".DB_PREFIX."countries ON ".DB_PREFIX."countries.coid = ".DB_PREFIX."orders.shipping_country
				WHERE 
					".DB_PREFIX."orders.oid='".$order_id."'
			");

			$db->moveNext();
			$data = $db->col;

			// set some environment variables
			$this->payment_status = $data['payment_status'];
			$this->custom = $data['custom1'];
			$this->transaction_id = $data['security_id'];
			
			$this->is_error = false;
			
			// set the transaction type to perform based on the $custom_id sent
			switch($custom_id)
			{
				// capture
				case '1':
					$transactionType = 'CustomerCreditCardCapture';
					$this->payment_status = 'Received';
					break;
				
				// Void transaction
				case '3':
					$transactionType = 'CustomerCreditCardTxnVoid';
					$this->payment_status = 'Canceled';
					break;
				
				default:
					$this->is_error = true;
					return false;
			}
			
			$buffer = $this->dispatch($transactionType, array(
					'transaction_id' => $this->transaction_id
				));
			
			if ($buffer)
			{
				$this->logStart();
				$this->log($buffer);
				$this->logFinish();
				
				$xmlParser = new XMLParser();
				
				$data = $xmlParser->parse($buffer);
				$data = $data['QBMSXML'][0]['QBMSXMLMsgsRs'][0]; 
				
				// preset some variables
				$rsStatusCode = $data[$transactionType.'Rs@'][0]['statusCode'];
				$rsStatusMessage = $data[$transactionType.'Rs@'][0]['statusMessage'];
				
				switch ($rsStatusCode)
				{
					case '0':
						// $this->createTransaction($db, $user, $order, $response_string, $rsClientTransID, $transactionType, "", $charged_amount, $rsCreditCardTransId);
						if ($transactionType == 'CustomerCreditCardCapture') {
							$rsAuthorizationCode = $data[$transactionType.'Rs'][0]['AuthorizationCode'][0];
							$rsPaymentStatus = $data[$transactionType.'Rs'][0]['PaymentStatus'][0];
							
							if ($rsPaymentStatus != 'Completed')
							{
								$this->is_error = true;
								$this->error_message = 'The transaction may have been successfully or unsuccessfully processed at the card issuer: there simply was no response (possibly due to network errors).';
							}
						}
						
						$rsClientTransID = $data[$transactionType.'Rs'][0]['ClientTransID'][0];
						$rsCreditCardTransId = $data[$transactionType.'Rs'][0]['CreditCardTransID'][0];
						
						// Update the orders table status
						$db->reset();
						$db->assignStr('payment_status', $this->payment_status);
						$db->assignStr("security_id", $rsCreditCardTransId);
						$db->assignStr("custom2", $buffer);
						$db->update(DB_PREFIX."orders", "WHERE oid='".$order_id."'");
						
						return true;
						break;
						
					// an error has occured
					default:
						$this->error_message = PAYMENT_TRANSACTION_ERROR_TEXT;
						return false;
				}
				
				
			}
			else
			{
				$this->is_error = true;
			}
			
			$this->error_message = 'The gateway could not process this request';
			
			// will return false
			return false;
		}
		
		/**
		 * undocumented function
		 *
		 * @param string $custom1 
		 * @param string $custom2 
		 * @param string $custom3 
		 * @param string $order_status 
		 * @param string $payment_status 
		 * @return void
		 * @author Sebastian Nievas
		 */
		function getVarStatus($custom1="", $custom2="", $custom3="", $order_status="", $payment_status="")
		{
			$var_status = array(1 => false, 2 => false, 3 => false);
			$message = '';
			if ($custom1 == "CustomerCreditCardAuth" && $payment_status == "Pending")
			{
				switch ($payment_status)
				{
					case 'Pending':
						$message = "
						<div style='padding-top:7px;'>
							<input type=\"button\" name=\"txn_action\" value=\"Complete Transaction\" onclick=\"window.open('content/engine/payment/qbms/qbms_gateway.php?oid=".$_GET['oid']."&payment_status=Pending','PopupWind','titlebar=no, toolbar=no, menubar=no, status=no, directories=no, resizable=no, scrollbars=no, top=200, left=400, width=600, height=150')\" />
						</div>";
						break;
				}
				
			}
			elseif($custom1 == 'CustomerCreditCardCharge' && $payment_status == 'Received')
			{
				return array();
			}
			
			$var_status[1] = array(
				'message' => $message,
				'executable' => true,
				'link_caption' => ''
			);
			
			return $var_status;
		}
		
		
		function dispatch($transactionType, $postVars)
		{
			global $settings;
			// production vars
			$applicationLogin = $this->settings_vars[$this->id."_app_login"]["value"];
			$appId = $this->settings_vars[$this->id."_app_id"]["value"];
			$connectionTicket = $this->settings_vars[$this->id."_connection_ticket"]["value"];
			
			$post_url = (trim($this->url_to_gateway) == '')
				? 'https://merchantaccount.quickbooks.com/j/AppGateway'
				: trim($this->url_to_gateway);
			
			// create unique transaction request id
			$transactionRequestId = md5(rand(1,9999).mktime());
			
			
			$input_array = array();
			switch ($transactionType)
			{
				case 'CustomerCreditCardAuth':
				case 'CustomerCreditCardCharge':
					
					if (mktime(0,0,0,date('m'),1,date('y')) > mktime(0,0,0,$postVars['cc_expiration_month'],1,$postVars['cc_expiration_year']))
					{
						$this->error_message = 'This credit card has expired. Please check the expiration date or use a different credit card.';
						return false;
					}
					
					$transactionRequestXml = '
						<CreditCardNumber>'.$postVars["cc_number"].'</CreditCardNumber>
						<ExpirationMonth>'.$postVars['cc_expiration_month'].'</ExpirationMonth>
						<ExpirationYear>'.$postVars['cc_expiration_year'].'</ExpirationYear>
						<IsECommerce>1</IsECommerce>
						<Amount>'.$this->common_vars["order_total_amount"]["value"].'</Amount>
						<NameOnCard>'.$postVars["cc_first_name"].' '.$postVars["cc_last_name"].'</NameOnCard>
						<CreditCardAddress>'.$this->common_vars["billing_address1"]["value"].' '.$this->common_vars["billing_address2"]["value"].'</CreditCardAddress>
						<CreditCardPostalCode>'.$this->common_vars["billing_zip"]["value"].'</CreditCardPostalCode>
						<SalesTaxAmount>'.$this->common_vars["order_tax_amount"]["value"].'</SalesTaxAmount>
						<CardSecurityCode>'.$postVars['cc_cvv2'].'</CardSecurityCode>';
					break;
					
				case 'CustomerCreditCardTxnVoid':
					
					
				case 'CustomerCreditCardCapture':
					$transactionRequestXml = '
						<CreditCardTransID>'.$postVars['transaction_id'].'</CreditCardTransID>
					';
					
					break;
				// not passing valid transactionType
				default: 
					return false;
			}
			
			$xml = '<?xml version="1.0"?>
				<?qbmsxml version="2.0"?>
				<QBMSXML>
					<SignonMsgsRq>
						<SignonDesktopRq>
							<ClientDateTime>'.date('Y-m-d\TH:i:s').'</ClientDateTime>
							<ApplicationLogin>'.$applicationLogin.'</ApplicationLogin>
							<ConnectionTicket>'.$connectionTicket.'</ConnectionTicket>
							<Language>English</Language>
							<AppID>'.$appId.'</AppID>
							<AppVer>1.0</AppVer>
						</SignonDesktopRq>
					</SignonMsgsRq>
					<QBMSXMLMsgsRq>
						<'.$transactionType.'Rq>
							<TransRequestID>'.$transactionRequestId.'</TransRequestID>
							'.$transactionRequestXml.'
						</'.$transactionType.'Rq>
					</QBMSXMLMsgsRq>
				</QBMSXML>
			';
			
			// initialize curl
			$ch = curl_init();
		
			$headers = array();
			$headers[] = "Content-type: application/x-qbmsxml";
			$headers[] = "Content-length: ".strlen($xml);

			
			if ($this->isTestMode())
			{
				$this->logStart();
				$this->log($xml);
				$this->logFinish();
			}
			
			curl_setopt($ch, CURLOPT_TIMEOUT, 500);
			curl_setopt($ch, CURLOPT_URL, $post_url);

			curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 1);
			curl_setopt($ch, CURLOPT_POSTFIELDS, $xml);
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
			curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);

			if( !ini_get('safe_mode') ) 
				set_time_limit(3000);
			
			curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 1);
			curl_setopt($ch, CURLOPT_CAPATH, $settings['GlobalServerPath'].$settings['SecuritySslDirectory']);
			curl_setopt($ch, CURLOPT_CAINFO, $settings['GlobalServerPath'].$settings['SecuritySslDirectory'].'/'.$settings['SecuritySslPem']);
			$buffer = curl_exec($ch);
			
			if (!$buffer)
			{
				$this->logStart();
				$this->log(curl_error($ch));
				$this->logFinish();
			}
			
			return $buffer;
		}
		
		/**
		 * Process user's payment
		 * @param $db
		 * @param $user
		 * @param $order
		 * @param $post_form
		 * @return unknown_type
		 */
		function process($db, $user, $order, $post_form)
		{
			global $settings;
			
			if (isset($post_form) && is_array($post_form))
			{
				
				// get the correct transaction request type
				switch ($this->settings_vars[$this->id.'_transaction_type']['value'])
				{
					case 'Authorize Only': $transactionType = 'CustomerCreditCardAuth'; break;
					case 'Authorize and Capture': $transactionType = 'CustomerCreditCardCharge'; break;
				}
				
				$buffer = $this->dispatch($transactionType, $post_form);
				
				$this->logStart();
				$this->log($buffer);
				$this->logFinish();
				
				/**
				 * Data received, parse it
				 */
				if ($buffer)
				{
					$xmlParser = new XMLParser();
					
					$data = $xmlParser->parse($buffer);
					
					$rsSignonMsgs = $data['QBMSXML'][0]['SignonMsgsRs'][0]['SignonDesktopRs@'];
					
					if ($rsSignonMsgs['statusCode'] > 0)
					{
						if ($this->isTestMode())
						{
							switch($rsSignonMsgs['statusCode'])
							{
								case '2000': $this->error_message = 'Authentication failed -- Invalid Connection Ticket'; break;
								case '2010': $this->error_message = 'Session Authentication Required'; break;
								case '2030': $this->error_message = 'Unsupported Signon Version'; break;
								case '2040': $this->error_message = 'Internal Error'; break;
							}
							
						} else {
							$this->error_message = PAYMENT_TRANSACTION_ERROR_TEXT;
						}
						
						$this->is_error = true;
						return false;
					}
					
					// we have some data
					$data = $data['QBMSXML'][0]['QBMSXMLMsgsRs'][0]; 
					
					// preset some variables
					$rsStatusCode = $data[$transactionType.'Rs@'][0]['statusCode'];
					$rsStatusMessage = $data[$transactionType.'Rs@'][0]['statusMessage'];
					
					// process the order based on status code (0 = successful)
					switch ($rsStatusCode)
					{
						case '0':
							$rsClientTransID = $data[$transactionType.'Rs'][0]['ClientTransID'][0];
							$rsCreditCardTransId = $data[$transactionType.'Rs'][0]['CreditCardTransID'][0];
							$charged_amount = $this->common_vars["order_total_amount"]["value"];

							// Create the response string
							$response = array();
							foreach ($data[$transactionType.'Rs'][0] as $key => $val)
							{
								$response[] = "$key = ".$val[0];
							}
							$response_string = implode(',', $response);
							
							// log the transaction
							$this->createTransaction($db, $user, $order, $response_string, $rsClientTransID, $transactionType, "", $charged_amount, $rsCreditCardTransId);

							// store credit card info?
							if($this->support_ccs && $this->enable_ccs){
								
								// PA DSS compliance
								$card_number = str_pad(substr($post_form['cc_number'], -4), strlen($post_form['cc_number']), 'x', STR_PAD_LEFT);
								
								$card_data = array(
									"fn"=>$post_form["cc_first_name"],
									"ln"=>$post_form["cc_last_name"],
									"ct"=>$post_form["cc_type"],
									"cc"=>$card_number,
									"em"=>$post_form["cc_expiration_month"],
									"ey"=>$post_form["cc_expiration_year"]
								);
								$this->saveCCdata($db, $order, $card_data, base64_decode($settings["SecurityCCSCertificate"]));
							}
							
							// transaction successful, return to not go past this point
							return ($transactionType == 'CustomerCreditCardAuth') ? 'Pending' : 'Received';
							
							break;
						
						// unknown error from gateway, print general text
						default: 
							
							
							if ($this->isTestMode())
							{
								$this->error_message = "Code {$rsStatusCode}: {$rsStatusMessage}<br />" . PAYMENT_TRANSACTION_ERROR_TEXT;
							} else {
								$this->error_message = PAYMENT_TRANSACTION_ERROR_TEXT;
							}
							
					} // end switch
					
					// if script reaches this point, there was an error from the gateway
					$this->is_error = true;
					return false;
					
				} else {
					// no $buffer set, return error
					$this->is_error = true;
					$this->error_message = ($this->error_message) ? $this->error_message : PAYMENT_SERVER_ERROR_TEXT;
				}
			}
			else{
				$this->is_error = true;
				$this->error_message = "Incorrect form params. Please contact site administrator.";
			}
			return !$this->is_error;
		}
	}